@@ -4,7 +4,7 @@
 # can do whatever you want with this stuff. If we meet some day, and you think
 # this stuff is worth it, you can buy me a beer in return.   Anton Berezin
 # ----------------------------------------------------------------------------
-# Copyright (c) 2005-2011 SPARTA, Inc.
+# Copyright (c) 2005-2013 SPARTA, Inc.
 # All rights reserved.
 #  
 # Redistribution and use in source and binary forms, with or without
@@ -33,7 +33,7 @@
 # OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 # ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
-# $Id: Fast.pm 5403 2011-02-15 00:25:58Z hardaker $
+# $Id: Fast.pm 7592 2013-02-20 15:59:02Z hardaker $
 #
 package Net::DNS::ZoneFile::Fast;
 # documentation at the __END__ of the file
@@ -46,16 +46,23 @@ use Net::DNS;
 use Net::DNS::RR;
 use MIME::Base64;
 
-$VERSION = '1.15';
+$VERSION = '1.19';
 
 my $MAXIMUM_TTL = 0x7fffffff;
 
 my $pat_ttl = qr{\d+[\dwdhms]*}i;
 my $pat_skip = qr{\s*(?:;.*)?};
 my $pat_name = qr{[-\*\w\$\d\/*]+(?:\.[-\*\w\$\d\/]+)*};
-my $pat_maybefullname = qr{[-\w\$\d\/*]+(?:\.[-\w\$\d\/]+)*\.?};
 my $pat_maybefullnameorroot = qr{(?:\.|[-\w\$\d\/*]+(?:\.[-\w\$\d\/]+)*\.?)};
 
+#
+# Added the ability to have a backslash in the SOA username.  This is to
+# provide for the RFC-allowed "Joe\.Jones.example.com" construct to allow
+# dots in usernames.  Keeping the original version here for easy reference.
+#
+# my $pat_maybefullname = qr{[-\w\$\d\/*]+(?:\.[-\w\$\d\/]+)*\.?};
+my $pat_maybefullname   = qr{[-\+\w\$\d\/*\\]+(?:\.[-\+\w\$\d\/]+)*\.?};
+
 my $debug;
 my $domain;
 my $parse;
@@ -71,6 +78,7 @@ my $sshfp;
 my $key;
 my $dnskey;
 my $ds;
+my $tlsa;
 my $on_error;
 my $quiet;
 my $soft_errors;
@@ -191,11 +199,14 @@ sub parse
 	      $z->{mbox} = uc $z->{mbox} if defined $z->{mbox};
 	      $z->{txtdname} = uc $z->{txtdname} if defined $z->{txtdname};
 	  }
-	  my $newrec = 
-	    Net::DNS::RR->new_from_hash(%$z);
+	  my $newrec = Net::DNS::RR->new(%$z);
 
 	  if ($newrec->{'type'} eq 'DNSKEY') {
-	      $newrec->setkeytag;
+	      if (ref($newrec) ne 'Net::DNS::RR::DNSKEY') {
+		  warn "Failed to define a DNSSEC object (got: " . ref($newrec) . "); you're probably missing either MIME::Base64 or MIME::Base32";
+	      } else {
+		  $newrec->setkeytag;
+	      }
 	  }
 
 	  # no longer an issue with recent Net::DNS
@@ -606,6 +617,33 @@ sub parse_line
 	  } else {
 	      error("bad txtdata in $type");
 	  }
+      } elsif (/\G(type[0-9]+)[ \t]+/igc) {
+          my $type = $1;
+	  if (/\G\\#\s+(\d+)\s+\(\s(.*)$/gc) {
+	      # multi-line
+	      $sshfp = { 
+			Line    => $ln,
+			name    => $domain,
+			type    => uc $type,
+			ttl     => $ttl,
+			class   => "IN",
+                        fptype  => $1,
+			fingerprint => $2,
+		       };
+	      $parse = \&parse_sshfp;
+	  } elsif (/\G\\#\s+(\d+)\s+(.*)$pat_skip$/gc) {
+	      push @zone, {
+			   Line    => $ln,
+			   name    => $domain,
+			   type    => uc $type,
+			   ttl     => $ttl,
+			   class   => "IN",
+                           fptype  => $1,
+			   fingerprint => $2,
+			  };
+	  } else {
+	      error("bad data in in $type");
+	  }
       } elsif (/\G(sshfp)[ \t]+/igc) {
 	  if (/\G(\d+)\s+(\d+)\s+\(\s*$/gc) {
 	      # multi-line
@@ -761,10 +799,9 @@ sub parse_line
 	  }
 
       } elsif (/\G(rrsig)[ \t]+/igc) {
-	  if (!/\G(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+/gc) {
-	      error("bad RRSIG data 1");
-	  }
-	  $rrsig = {
+	  if (/\G(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+/gc) {
+	      # some versions of bind (>=10) put the sig-expir on the first line
+	      $rrsig = {
 		    first     => 1,
 		    Line      => $ln,
 		    name      => $domain,
@@ -775,8 +812,19 @@ sub parse_line
 		    algorithm => $2,
 		    labels    => $3,
 		    orgttl    => $4,
-		    sigexpiration => $5
 		   };
+	  } else {
+	      error("bad RRSIG data 1");
+	  }
+
+	  if (/\G(\d+)\s+/gc) {
+	      # some versions of bind (<10) put the sig-expir on the first line
+	      # and newer ones put it on the next.
+	      $rrsig->{'sigexpiration'} = $1;
+	  } else {
+	      $rrsig->{'needsigexp'} = $1;
+	  }
+
 	  if (/\G\(\s*$/gc) {
 	      # multi-line
 	      $parse = \&parse_rrsig;
@@ -842,12 +890,46 @@ sub parse_line
 	      $ds->{'digest'} .= $1;
 	      $ds->{'digest'} = lc($ds->{'digest'});
 	      $ds->{'digest'} =~ s/\s//g;
+	      # remove any surrounding single line ()s
+	      $ds->{'digest'} =~ s/^\(//;
+	      $ds->{'digest'} =~ s/\)$//;
 	      $ds->{'digestbin'} = pack("H*", $ds->{'digest'});
 	      push @zone, $ds;
 	      $ds = undef;
 	  } else {
 	      error("bad DS data");
 	  }
+      } elsif (/\G(tlsa)[ \t]+/igc) {
+	  if (!/\G(\d+)\s+(\d+)\s+(\d+)\s+/gc) {
+	      error("bad TLSA data 1");
+	  }
+	  $tlsa = {
+		 Line      => $ln,
+		 name      => $domain,
+		 class     => "IN",
+		 ttl       => $ttl,
+		 type      => "TLSA",
+		 usage     => $1,
+		 selector  => $2,
+		 matchingtype => $3,
+		};
+	  if (/\G\(\s*$/gc) {
+	      # multi-line
+	      $parse = \&parse_tlsa;
+	  } elsif (/\G(.*\S)\s*$/) {
+	      # single line
+	      $tlsa->{'cert'} .= $1;
+	      $tlsa->{'cert'} = lc($tlsa->{'cert'});
+	      $tlsa->{'cert'} =~ s/\s//g;
+	      # remove any surrounding single line ()s
+	      $tlsa->{'cert'} =~ s/^\(//;
+	      $tlsa->{'cert'} =~ s/\)$//;
+	      $tlsa->{'certbin'} = pack("H*", $tlsa->{'cert'});
+	      push @zone, $tlsa;
+	      $tlsa = undef;
+	  } else {
+	      error("bad TLSA data");
+	  }
       } elsif (/\G(nsec)[ \t]+/igc) {
 	  if (/\G\s*($pat_maybefullnameorroot)\s+(.*?)$pat_skip$/gc) {
 	      # XXX: set the typebm field ourselves?
@@ -981,7 +1063,7 @@ sub parse_line
       } elsif (/\Gany\s+tsig.*$/igc) {
 	  # XXX ignore tsigs
       } else {
-	  error("unrecognized type");
+	  error("unrecognized type for $domain\n$_\n");
       }
   }
 
@@ -1063,7 +1145,15 @@ sub parse_rrsig
       # got more data
       if ($rrsig->{'first'}) {
 	  delete $rrsig->{'first'};
-	  if (/\G\s*(\d+)\s+(\d+)\s+($pat_maybefullnameorroot)/gc) {
+	  if (exists($rrsig->{'needsigexp'}) &&
+	      /\G\s*(\d+)\s+(\d+)\s+(\d+)\s+($pat_maybefullnameorroot)/gc) {
+	      delete $rrsig->{'needsigexp'};
+	      $rrsig->{'sigexpiration'} = $1;
+	      $rrsig->{'siginception'} = $2;
+	      $rrsig->{'keytag'} = $3;
+	      $rrsig->{'signame'} = $4;
+	  } elsif (!exists($rrsig->{'needsigexp'}) && 
+		   /\G\s*(\d+)\s+(\d+)\s+($pat_maybefullnameorroot)/gc) {
 	      $rrsig->{'siginception'} = $1;
 	      $rrsig->{'keytag'} = $2;
 	      $rrsig->{'signame'} = $3;
@@ -1167,6 +1257,32 @@ sub parse_ds
       }
   }
 
+sub parse_tlsa
+  {
+      # got more data
+      if (/\)\s*$/) {
+	  if (/\G\s*(\S*)\s*\)\s*$/gc) {
+	      $tlsa->{'cert'} .= $1;
+	      $tlsa->{'cert'} = lc($tlsa->{'cert'});
+
+	      # we're done
+	      $parse = \&parse_line;
+
+	      $tlsa->{'certbin'} = pack("H*",$tlsa->{'cert'});
+	      push @zone, $tlsa;
+	      $tlsa = undef;
+	  } else {
+	      error("bad tlsa last line");
+	  }
+      } else {
+	  if (/\G\s*(\S+)\s*$/gc) {
+	      $tlsa->{'cert'} .= $1;
+	  } else {
+	      error("bad tlsa remaining lines");
+	  }
+      }
+  }
+
 sub parse_soa_number
   {
       error("parse_soa_number: internal error, no \$soa") unless $soa;
@@ -1466,7 +1582,7 @@ Copyright 2003 by Anton Berezin and catpipe Systems ApS
 
   Anton Berezin
 
-Copyright (c) 2004-2011 SPARTA, Inc.
+Copyright (c) 2004-2013 SPARTA, Inc.
   All rights reserved.
    
   Redistribution and use in source and binary forms, with or without
