@@ -1,5 +1,5 @@
 #
-# Copyright 2006-2011 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2006-2013 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details
 #
 # DNSSEC Tools
@@ -25,83 +25,105 @@ our @EXPORT = qw(
 		   dnssec_tools_defnames
 		);
 
-our $VERSION = "1.9";
-our $MODULE_VERSION = "1.9.0";
+our $VERSION = "2.0";
+our $MODULE_VERSION = "2.0.0";
 
 my $installdir = getprefixdir() . "/bin"; # DNSSEC-Tools installation directory.
 
 my %defaults =
 (
-	"admin-email"	   => "root",		# Admin's email address.
-	"archivedir"	   => getprefixdir() . "/var/key-archive",
-	"algorithm"	   => "rsasha1",	# Encryption algorithm.
-	"enddate"	   => "+2592000",	# Zone life, in seconds.
-	"entropy_msg"	   => 1,		# Display entropy message flag.
-	"keygen"	   => getprefixdir() . "/sbin/dnssec-keygen",
-	"keygen-opts"	   => "",		# Options for key generator.
-	"kskcount"	   => 1,		# Number of KSK keys.
-	"ksklength"	   => 2048,		# Length of KSK key.
-	"ksklife"	   => 15768000,		# Lifespan of KSK key.
-	"revperiod"	   => 3888000,		# Revocation period.
-	"lifespan-max"	   => 94608000,		# Max lifespan (two years.)
-	"lifespan-min"	   => 3600,		# Min lifespan (one hour.)
-	"random"	   => "/dev/urandom",	# Random no. generator device.
-	"rndc"		   => getprefixdir() . "/sbin/rndc",
-	"roll_loadzone"	   => 1,		# Zone-reloading flag.
-	"roll_logfile"	   => makelocalstatedir() . "/log.rollerd",
-	"roll_loglevel"	   => "phase",		# Rollerd's logging level.
-	"roll_phasemsg"	   => "long",		# Rollerd's phase logmsg length.
-	"roll_sleeptime"   => 3600,		# Rollerd's sleep time.
-	"log_tz"	   => "gmt",		# Log message timezone.
-	"savekeys"	   => 1,		# Save/delete old keys flag.
-	"tacontact" 	   => "",
-	"tatmpdir" 	   => "/tmp/dnssec-tools/trustman",
-	"tadnsvalconffile" => getconfdir() . "/dnsval.conf",
-	"tanamedconffile"  => getconfdir() . "/named/named.conf",
-	"tasleeptime"      => 3600,
-	"tasmtpserver"     => "localhost",	# Trustman's SMTP server.
-	"taresolvconf"	   => "/etc/resolv.conf",	# resolv.conf file.
-	"usegui"	   => 0,		# Use GUI for option entry flag.
-#	"viewimage"	   => "/usr/X11R6/bin/viewimage",
-	"zone_errors"	   => 5,
-	"zonecheck"	   => getprefixdir() . "/sbin/named-checkzone",
-	"zonecheck-opts"   => "",		# Options for zone checker.
-	"zonesign"	   => getprefixdir() . "/sbin/dnssec-signzone",
-	"zonesign-opts"	   => "",		# Options for zone signer.
-	"zskcount"	   => 1,		# Number of Current ZSK keys.
-	"zsklength"	   => 1024,		# Length of ZSK key.
-	"zsklife"	   => 604800,		# Lifespan of ZSK key.
-	"usensec3"         => "no",             # Use NSEC3 by default
-	"nsec3iter"	   => 100,              # default NSEC3 iterations
-	"nsec3salt"	   => "random:64",      # default NSEC3 salt
-	"nsec3optout"	   => "no",             # NSEC3 opt-out default
-	"blinkenlights"	   => "$installdir/blinkenlights",
-	"cleanarch"	   => "$installdir/cleanarch",
-	"cleankrf"	   => "$installdir/cleankrf",
-	"dtconf"	   => "$installdir/dtconf",
-	"dtconfchk"	   => "$installdir/dtconfchk",
-	"dtdefs"	   => "$installdir/dtdefs",
-	"dtinitconf"	   => "$installdir/dtinitconf",
-	"expchk"	   => "$installdir/expchk",
-	"fixkrf"	   => "$installdir/fixkrf",
-	"genkrf"	   => "$installdir/genkrf",
-	"getdnskeys"	   => "$installdir/getdnskeys",
-	"keyarch"	   => "$installdir/keyarch",
-	"krfcheck"	   => "$installdir/krfcheck",
-	"lskrf"		   => "$installdir/lskrf",
-	"lsroll"	   => "$installdir/lsroll",
-	"rollchk"	   => "$installdir/rollchk",
-	"rollctl"	   => "$installdir/rollctl",
-	"rollerd"	   => "$installdir/rollerd",
-	"rollinit"	   => "$installdir/rollinit",
-	"rolllog"	   => "$installdir/rolllog",
-	"rollrec-editor"   => "$installdir/rollrec-editor",
-	"rollset"	   => "$installdir/rollset",
-	"signset-editor"   => "$installdir/signset-editor",
-	"tachk"		   => "$installdir/tachk",
-	"timetrans"	   => "$installdir/timetrans",
-	"trustman"	   => "$installdir/trustman",
-	"zonesigner"	   => "$installdir/zonesigner",
+	'admin-email'	   => "root",		# Admin's email address.
+	'archivedir'	   => getprefixdir() . "/var/key-archive",
+	'algorithm'	   => "rsasha1",	# Encryption algorithm.
+	'autosign'	   => 1,		# Auto-sign zone files flag.
+	'enddate'	   => "+2764800",	# Zone life, in seconds.
+	'entropy_msg'	   => 1,		# Display entropy message flag.
+	'keygen'	   => getprefixdir() . "/sbin/dnssec-keygen",
+	'keygen-opts'	   => "",		# Options for key generator.
+	'kskcount'	   => 1,		# Number of KSK keys.
+	'ksklength'	   => 2048,		# Length of KSK key.
+	'ksklife'	   => 31536000,		# Lifespan of KSK key.
+	'revperiod'	   => 3888000,		# Revocation period.
+	'lifespan-max'	   => 94608000,		# Max lifespan (two years.)
+	'lifespan-min'	   => 3600,		# Min lifespan (one hour.)
+	'prog-ksk1'	   => 'default',	# Program for KSK phase 1.
+	'prog-ksk2'	   => 'default',	# Program for KSK phase 2.
+	'prog-ksk3'	   => 'default',	# Program for KSK phase 3.
+	'prog-ksk4'	   => 'default',	# Program for KSK phase 4.
+	'prog-ksk5'	   => 'default',	# Program for KSK phase 5.
+	'prog-ksk6'	   => 'default',	# Program for KSK phase 6.
+	'prog-ksk7'	   => 'default',	# Program for KSK phase 7.
+	'prog-normal'	   => 'default',	# Program for non-rollover.
+	'prog-zsk1'	   => 'default',	# Program for ZSK phase 1.
+	'prog-zsk2'	   => 'default',	# Program for ZSK phase 2.
+	'prog-zsk3'	   => 'default',	# Program for ZSK phase 3.
+	'prog-zsk4'	   => 'default',	# Program for ZSK phase 4.
+	'random'	   => "/dev/urandom",	# Random no. generator device.
+	'rndc'		   => getprefixdir() . "/sbin/rndc",
+	'roll_loadzone'	   => 1,		# Zone-reloading flag.
+	'roll_logfile'	   => makelocalstatedir() . "/log.rollerd",
+	'roll_loglevel'	   => "phase",		# Rollerd's logging level.
+	'roll_phasemsg'	   => "long",		# Rollerd's phase logmsg length.
+	'roll_sleeptime'   => 3600,		# Rollerd's sleep time.
+	'log_tz'	   => "gmt",		# Log message timezone.
+	'savekeys'	   => 1,		# Save/delete old keys flag.
+	'mailer-server'    => "localhost",	# Mail server.
+	'mailer-type'      => "smtp",		# Mail type.
+	'tacontact' 	   => "",
+	'tatmpdir' 	   => "/tmp/dnssec-tools/trustman",
+	'tadnsvalconffile' => getconfdir() . "/dnsval.conf",
+	'tanamedconffile'  => getconfdir() . "/named/named.conf",
+	'tasleeptime'      => 3600,
+	'tasmtpserver'     => "localhost",	# Trustman's SMTP server.
+	'taresolvconf'	   => "/etc/resolv.conf",	# resolv.conf file.
+	'usegui'	   => 0,		# Use GUI for option entry flag.
+	'zone_errors'	   => 5,
+	'zonecheck'	   => getprefixdir() . "/sbin/named-checkzone",
+	'zonecheck-opts'   => "-i local",	# Options for zone checker.
+	'zonefile-parser'  => "Net::DNS::ZoneFile::Fast",
+	'zonesign'	   => getprefixdir() . "/sbin/dnssec-signzone",
+	'zonesign-opts'	   => "",		# Options for zone signer.
+	'zskcount'	   => 1,		# Number of Current ZSK keys.
+	'zsklength'	   => 1024,		# Length of ZSK key.
+	'zsklife'	   => 7884000,		# Lifespan of ZSK key.
+	'usensec3'         => "no",             # Use NSEC3 by default
+	'nsec3iter'	   => 100,              # default NSEC3 iterations
+	'nsec3salt'	   => "random:64",      # default NSEC3 salt
+	'nsec3optout'	   => "no",             # NSEC3 opt-out default
+	'blinkenlights'	   => "$installdir/blinkenlights",
+	'cleanarch'	   => "$installdir/cleanarch",
+	'cleankrf'	   => "$installdir/cleankrf",
+	'dtconf'	   => "$installdir/dtconf",
+	'dtconfchk'	   => "$installdir/dtconfchk",
+	'dtdefs'	   => "$installdir/dtdefs",
+	'dtinitconf'	   => "$installdir/dtinitconf",
+	'dtrealms'	   => "$installdir/dtrealms",
+	'expchk'	   => "$installdir/expchk",
+	'fixkrf'	   => "$installdir/fixkrf",
+	'genkrf'	   => "$installdir/genkrf",
+	'getdnskeys'	   => "$installdir/getdnskeys",
+	'grandvizier'	   => "$installdir/grandvizier",
+	'keyarch'	   => "$installdir/keyarch",
+	'krfcheck'	   => "$installdir/krfcheck",
+	'lsdnssec'	   => "$installdir/lsdnssec",
+	'lskrf'		   => "$installdir/lskrf",
+	'lsrealm'	   => "$installdir/lsrealm",
+	'lsroll'	   => "$installdir/lsroll",
+	'realmchk'	   => "$installdir/realmchk",
+	'realmctl'	   => "$installdir/realmctl",
+	'realminit'	   => "$installdir/realminit",
+	'rollchk'	   => "$installdir/rollchk",
+	'rollctl'	   => "$installdir/rollctl",
+	'rollerd'	   => "$installdir/rollerd",
+	'rollinit'	   => "$installdir/rollinit",
+	'rolllog'	   => "$installdir/rolllog",
+	'rollrec-editor'   => "$installdir/rollrec-editor",
+	'rollset'	   => "$installdir/rollset",
+	'signset-editor'   => "$installdir/signset-editor",
+	'tachk'		   => "$installdir/tachk",
+	'timetrans'	   => "$installdir/timetrans",
+	'trustman'	   => "$installdir/trustman",
+	'zonesigner'	   => "$installdir/zonesigner",
 );
 
 #--------------------------------------------------------------------------
@@ -248,13 +270,6 @@ This default holds the default lifespan of a KSK key.  This is only used
 for determining when to rollover the KSK key.  Keys otherwise have no
 concept of a lifespan.  This is measured in seconds.
 
-=item B<revperiod>
-
-This default holds the default revocation period of a KSK key. This is
-the minimum period of time a revoked KSK is required to remain in the
-signing set so that it is properly observed by resolvers.  This is
-measured in seconds.
-
 =item B<lifespan-max>
 
 This default is the maximum lifespan of a key.
@@ -267,10 +282,52 @@ This default is the minimum lifespan of a key.
 
 This default is the timezone to be used in log-message timestamps.
 
+=item B<mailer-server>
+
+The mail server that will be contacted by I<dt_adminmail()>.  This is
+passed to I<Mail::Send>.
+
+=item B<mailer-type>
+
+The type of mailer that will be contacted by I<dt_adminmail()>.  This is
+passed to I<Mail::Mailer> (by way of I<Mail::Send>.)  Any values recognized
+by I<Mail::Mailer> may be used here.
+
+=item B<prog_ksk1> ... B<prog_ksk7>
+
+These defaults hold the default phase commands to be executed by B<rollerd>
+for each phase of KSK rollover.  The I<default> keyword indicates that the
+normal phase processing should be performed.  Multiple commands may be given,
+but they must be separated by bangs.  The I<default> keyword may be
+combined with other commands.
+
+=item B<prog_normal>
+
+These defaults hold the default phase commands to be executed by B<rollerd>
+when a zone is not in a rollover state.  The I<default> keyword indicates that
+the normal phase processing should be performed.  Multiple commands may be
+given, but they must be separated by bangs.  The I<default> keyword may
+be combined with other commands.
+
+=item B<prog_zsk1> ... B<prog_zsk7>
+
+These defaults hold the default phase commands to be executed by B<rollerd>
+for each phase of ZSK rollover.  The I<default> keyword indicates that the
+normal phase processing should be performed.  Multiple commands may be given,
+but they must be separated by bangs.  The I<default> keyword may be
+combined with other commands.
+
 =item B<random>
 
 This default holds the default random number generator device.
 
+=item B<revperiod>
+
+This default holds the default revocation period of a KSK key. This is
+the minimum period of time a revoked KSK is required to remain in the
+signing set so that it is properly observed by resolvers.  This is
+measured in seconds.
+
 =item B<rndc>
 
 This default is the default path of the BIND B<rndc> program.
@@ -345,6 +402,14 @@ This default holds the path to the zone-verification program.
 
 This default hold a set of options for the zone-verification program.
 
+This default is set to "-i local".  This value has been found to greatly
+improve the amount of time it takes B<named-checkzone> to run.
+
+=item B<zonefile-parser>
+
+This default specifies the parser that will be used to parse zone files.
+The default value is to use the B<Net::DNS::ZoneFile::Fast> module.
+
 =item B<zonesign>
 
 This default holds the path to the zone-signing program.
@@ -488,11 +553,11 @@ This default holds the path to the DNSSEC-Tools B<zonesigner> program.
 
 =head1 COPYRIGHT
 
-Copyright 2006-2011 SPARTA, Inc.  All rights reserved.
+Copyright 2006-2013 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
 
-Wayne Morrison, tewok@users.sourceforge.net
+Wayne Morrison, tewok@tislabs.com
 
 =cut
