@@ -1,5 +1,5 @@
 #
-# Copyright 2005-2011 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2005-2013 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details
 #
 # DNSSEC-Tools
@@ -28,7 +28,7 @@
 #			zonefile	"db.example.com"
 #			zskcur		"signing-set-42"
 #			zskpub		"signing-set-43"
-#			endtime		"+2592000"   # Zone expires in 30 days.
+#			endtime		"+2764800"   # Zone expires in 32 days.
 #
 #		set "signing-set-42"
 #			zonename	"example.com"
@@ -59,8 +59,10 @@ use strict;
 use Net::DNS::SEC::Tools::conf;
 use Fcntl qw(:DEFAULT :flock);
 
-our $VERSION = "1.9";
-our $MODULE_VERSION = "1.9.0";
+our $VERSION = "2.0";
+our $MODULE_VERSION = "2.0.0";
+
+#--------------------------------------------------------------------------
 
 our @ISA = qw(Exporter);
 
@@ -74,29 +76,33 @@ our @EXPORT = qw(keyrec_creat keyrec_open keyrec_read
 		 keyrec_write keyrec_saveas keyrec_keypaths
 		 keyrec_curkrf keyrec_defkrf
 		 keyrec_dump_hash keyrec_dump_array
-		 keyrec_signset_newname keyrec_signset_new
+		 keyrec_signset_newname keyrec_signset_new keyrec_signset_keys
 		 keyrec_signset_addkey keyrec_signset_delkey
 		 keyrec_signset_haskey keyrec_signset_clear keyrec_signsets
+		 keyrec_signset_prefix
 		 keyrec_fmtchk
 	      );
 
+#--------------------------------------------------------------------------
+
 #
 # Fields in a zone keyrec.
 #
 my @ZONEFIELDS = (
-			'zonefile',
 			'keyrec_type',
-			'endtime',
+			'zonefile',
+			'signedzone',
 			'zskdirectory',
 			'kskdirectory',
 			'archivedir',
+			'dsdir',
 			'ksdir',
+			'endtime',
 			'kskcount',
 			'kskcur',
 			'kskpub',
 			'kskrev',
 	                'rfc5011',
-			'signedzone',
 			'zskcount',
 			'zskcur',
 			'zskpub',
@@ -105,6 +111,16 @@ my @ZONEFIELDS = (
 			'lastset',
 			'gends',
 			'szopts',
+			'rollmgr',
+			'lastcmd',
+			'new_kgopts',
+			'new_ksklength',
+			'new_ksklife',
+			'new_random',
+			'new_revperiod',
+			'new_revtime',
+			'new_zsklength',
+			'new_zsklife',
 			'keyrec_signsecs',
 			'keyrec_signdate',
 		  );
@@ -141,6 +157,10 @@ my @KEYFIELDS = (
 		        'zonename',
 		 );
 
+#--------------------------------------------------------------------------
+
+my $DEFSETPREFIX = '-signset-';		# Default signing-set prefix.
+
 my $curkrfname;				# Name of open keyrec file.
 
 my @keyreclines;			# Keyrec lines.
@@ -509,7 +529,7 @@ sub keyrec_names
 # Routine:	keyrec_keypaths()
 #
 # Purpose:	Return an array of the paths of a zone keyrec's keys.
-#		The type of key is specified by the caller.
+#		The type of key to return is specified by the caller.
 #
 sub keyrec_keypaths
 {
@@ -520,6 +540,11 @@ sub keyrec_keypaths
 	my $keylist;				# List of keys.
 	my @paths = ();				# Array for keyrec names.
 
+	#
+	# Ensure the key type is in the expected case.
+	#
+	$krt = lc($krt);
+
 	#
 	# Ensure this is a zone keyrec.
 	#
@@ -531,44 +556,140 @@ sub keyrec_keypaths
 	if(($krt ne "kskcur") && ($krt ne "kskpub") &&
 	   ($krt ne "kskrev") && ($krt ne "kskobs") &&
 	   ($krt ne "zskcur") && ($krt ne "zskpub") &&
-	   ($krt ne "zsknew") && ($krt ne "zskobs"))
+	   ($krt ne "zsknew") && ($krt ne "zskobs") &&
+	   ($krt ne "ksk")    && ($krt ne "zsk")    && ($krt ne "all"))
 	{
 		return(@paths);
 	}
 
 	#
-	# Get and verify the name of the appropriate signing set.
+	# Handle the special keytypes.
 	#
-	return(@paths) if(!defined($keyrecs{$krn}{$krt}));
-	$sset = $keyrecs{$krn}{$krt};
+	if($krt eq "all")
+	{
+		@paths = keyrec_keypaths($krn,"ksk");
+		@paths = (@paths, keyrec_keypaths($krn,"zsk"));
+		return(@paths);
+	}
+	elsif($krt eq "ksk")
+	{
+		@paths = keyrec_keypaths($krn,"kskcur");
+		@paths = (@paths, keyrec_keypaths($krn,"kskpub"));
+		@paths = (@paths, keyrec_keypaths($krn,"kskrev"));
+		@paths = (@paths, keyrec_keypaths($krn,"kskobs"));
+		return(@paths);
+	}
+	elsif($krt eq "zsk")
+	{
+		@paths = keyrec_keypaths($krn,"zskcur");
+		@paths = (@paths, keyrec_keypaths($krn,"zskpub"));
+		@paths = (@paths, keyrec_keypaths($krn,"zsknew"));
+		@paths = (@paths, keyrec_keypaths($krn,"zskobs"));
+		return(@paths);
+	}
 
 	#
-	# Get and verify the key list.
+	# Get and verify the name of the appropriate signing set.
+	# We have to handle obsolete keys differently than other keys
+	# since obsolete keys don't have a reference in the zone keyrec.
 	#
-	return(@paths) if(!defined($keyrecs{$sset}{'keys'}));
-	$keylist = $keyrecs{$sset}{'keys'};
+	if(($krt ne 'kskobs') && ($krt ne 'zskobs'))
+	{
+		return(@paths) if(!defined($keyrecs{$krn}{$krt}));
+
+		#
+		# Get and verify the key list.
+		#
+		$sset = $keyrecs{$krn}{$krt};
+
+		return(@paths) if(!defined($keyrecs{$sset}{'keys'}));
+		$keylist = $keyrecs{$sset}{'keys'};
+	}
+	else
+	{
+		foreach my $kname (keyrec_names())
+		{
+			next if($keyrecs{$kname}{'set_type'} ne $krt);
+			$keylist .= " $keyrecs{$kname}{'keys'}";
+		}
+	}
 
 	#
 	# Get the keys' paths and add 'em to the path array.
 	#
-	foreach my $kn (split /[\s,]/, $keylist)
+	foreach my $kn (sort(split /[\s,]/, $keylist))
 	{
-		#
-		# Verify that this key exists and is the right type.
-		#
-		next if(!defined($keyrecs{$kn}));
-		next if($keyrecs{$kn}{'keyrec_type'} ne $krt);
+		my @newpaths;
 
-		#
-		# Push the key's path onto the path list.
-		#
-		push @paths, $keyrecs{$kn}{'keypath'};
+		@newpaths = getsetpaths($kn,$krt);
+		push @paths, @newpaths if(@newpaths != 0);
 	}
 
 	#
 	# Return the path list.
 	#
-	return(sort(@paths));
+	return(@paths);
+}
+
+#--------------------------------------------------------------------------
+# Routine:	getsetpaths()
+#
+# Purpose:	Build a list of the keypaths for a given set.  This will
+#		fetch the paths for intermediate signing sets.
+#
+#		This routine is not exported.
+#
+sub getsetpaths
+{
+	my $kn = shift;					# Name of key to get.
+	my $kt = shift;					# Type of key to get.
+	my @paths = ();				# Set's path list.
+
+	#
+	# Verify that this key exists.
+	#
+	return(@paths) if($kn eq '') || (! defined($keyrecs{$kn}));
+
+	#
+	# If this is a set keyrec, we'll pull the key paths from its keylist.
+	# Otherwise, it's assumed to be a key keyrec and we'll take the key
+	# path from itself.
+	#
+	if($keyrecs{$kn}{'keyrec_type'} eq 'set')
+	{
+		my $keylist = $keyrecs{$kn}{'keys'};
+
+		#
+		# Get the keys' paths and add 'em to the path array.
+		#
+		foreach my $kname (split /[\s,]/, $keylist)
+		{
+			my @newpaths;
+			@newpaths = getsetpaths($kname,$kt);
+			push @paths, @newpaths if(@newpaths != 0);
+		}
+
+	}
+	else
+	{
+		#
+		# Verify that this key is the right type.
+		#
+		return(@paths) if($keyrecs{$kn}{'keyrec_type'} ne $kt);
+
+		#
+		# Push the key's path onto the path list.
+		#
+		if($keyrecs{$kn}{'keypath'} ne '')
+		{
+			push @paths, $keyrecs{$kn}{'keypath'};
+		}
+	}
+
+	#
+	# Return the path list.
+	#
+	return(@paths);
 }
 
 #--------------------------------------------------------------------------
@@ -758,7 +879,6 @@ sub keyrec_setval
 		}
 	}
 
-	
 	#
 	# If we found the entry, we'll modify it in place.
 	# If we didn't find the entry, we'll insert a new line into the array.
@@ -769,17 +889,23 @@ sub keyrec_setval
 	}
 	else
 	{
-		my $newline = "\t$field\t\t\"$val\"\n";
+		my $newline;			# New keyrec field to save.
+		my @endarr;			# Last part of @keyreclines.
 
 		#
-		# If the keyword is longer than 7 characters, we'll lop out one
-		# of the tabs between the keyword and the value.  This is to do
-		# some rough, simple formatting to keep the keyrec file somewhat
-		# orderly.  This assumes eight-character tabstops.
+		# Create the new line.  If the keyword is longer than 7
+		# characters, we'll only use a single tab between the
+		# keyword and the value.  This is to do some rough, simple
+		# formatting to keep the keyrec file somewhat orderly.
+		# This assumes eight-character tabstops.
 		#
 		if(length($field) > 7)
 		{
-			$newline =~ s/\t\t/\t/;
+			$newline = "\t$field\t\"$val\"\n";
+		}
+		else
+		{
+			$newline = "\t$field\t\t\"$val\"\n";
 		}
 
 		#
@@ -788,7 +914,8 @@ sub keyrec_setval
 		# in the middle, we'll do the magic to insert it at the start
 		# of the keyrec.
 		#
-		my @endarr = splice(@keyreclines,$krind+1);
+		$lastfld++ if($keyreclines[$lastfld] eq "\n");
+		@endarr = splice(@keyreclines,$lastfld+1);
 		push @keyreclines, $newline;
 		push @keyreclines, @endarr;
 
@@ -1339,8 +1466,7 @@ sub keyrec_signset_newname
 	my $setname;				# Name of zone's last set.
 	my $oldind;				# Old index of zone's last set.
 	my $newind;				# New index of zone's last set.
-
-	my $setprefix = '-signset-';
+	my $ssprefix;				# Signing-set prefix.
 
 # print "keyrec_signset_newname($zone):  down in\n";
 
@@ -1350,11 +1476,16 @@ sub keyrec_signset_newname
 	$setname = $keyrecs{$zone}{'lastset'};
 	$setname = "$zone-signset-000" if(!defined($keyrecs{$zone}{'lastset'}));
 
+	#
+	# Build the signing-set prefix.
+	#
+	$ssprefix = keyrec_signset_prefix($zone);
+
 	#
 	# Get the first number in the set name.  If there isn't a number
 	# in the set name, we'll append a zero.
 	#
-	$setname =~ /$setprefix(\d+)/;
+	$setname =~ /$ssprefix(\d+)/;
 	$oldind = $1;
 	
 	if($oldind eq '')
@@ -1373,7 +1504,7 @@ sub keyrec_signset_newname
 		$newind = "0$newind";
 	}
 
-	$setname =~ s/$setprefix$oldind/$setprefix$newind/;
+	$setname =~ s/$ssprefix$oldind/$ssprefix$newind/;
 	keyrec_setval('zone',$zone,'lastset',$setname);
 
 	#
@@ -1667,6 +1798,71 @@ sub keyrec_signsets
 	return(@signset);
 }
 
+#--------------------------------------------------------------------------
+# Routine:	keyrec_signsets_keys()
+#
+# Purpose:	Return the names of the keys in a specified signing set.
+#
+sub keyrec_signset_keys
+{
+	my $krname = shift;		# Keyrec name to examine.
+	my $sstype = shift;		# Signing set type to expand.
+	my $error  = '';		# Error return.
+
+	#
+	# Ensure the named keyrec exists.
+	#
+	return(undef) if(!defined($keyrecs{$krname}));
+
+	#
+	# If the named keyrec is a zone keyrec, we'll get the signing
+	# set keyrec specified in the second argument.
+	#
+	if($keyrecs{$krname}{'keyrec_type'} eq 'zone')
+	{
+		#
+		# Ensure we were given a proper signing set type.
+		#
+		if(($sstype ne 'kskcur') && ($sstype ne 'kskpub') &&
+		   ($sstype ne 'kskrev') && ($sstype ne 'kskobs') &&
+		   ($sstype ne 'zskcur') && ($sstype ne 'zskpub') &&
+		   ($sstype ne 'zsknew') && ($sstype ne 'zskobs'))
+		{
+			return(undef);
+		}
+
+		#
+		# Use the signing set name instead of the zone name.
+		#
+		$krname = $keyrecs{$krname}{$sstype};
+	}
+
+	#
+	# Ensure the supposed signing set is actually a signing set.
+	#
+	return(undef) if($keyrecs{$krname}{'keyrec_type'} ne 'set');
+
+	#
+	# Return the signing set's key names.
+	#
+	return($keyrecs{$krname}{'keys'});
+}
+
+#--------------------------------------------------------------------------
+# Routine:	keyrec_signset_prefix()
+#
+# Purpose:	Build and return the signing-set prefix.
+#
+sub keyrec_signset_prefix
+{
+	my $zone = shift;			# Zone for this signing set.
+	my $prefix;				# Signing-set prefix.
+
+	$prefix = $zone . $DEFSETPREFIX;
+
+	return($prefix);
+}
+
 #--------------------------------------------------------------------------
 # Routine:	keyrec_init()
 #
@@ -2216,6 +2412,8 @@ Net::DNS::SEC::Tools::keyrec - DNSSEC-Tools I<keyrec> file operations
 
   @signset = keyrec_signsets();
 
+  $sset_prefix = keyrec_signset_prefix("example.com");
+
   keyrec_settime("zone","example.com");
   keyrec_settime("set","signing-set-42");
   keyrec_settime("key","Kexample.com.+005+76543");
@@ -2416,10 +2614,17 @@ given zone.  The zone is specified in I<zonename> and the type of key is
 given in I<keytype>.
 
 I<keytype> must be one of the following:  "kskcur", "kskpub", "kskrev",
-"kskobs"", "zskcur", "zskpub", "zsknew", or "zskobs".
+"kskobs"", "zskcur", "zskpub", "zsknew", "zskobs", "ksk", "zsk", or "all".
+Case does not matter for the I<keytype>.
 
-If the given key type is not defined in the given zone's zone I<keyrec>, then
-a null set is returned.
+If I<keytype> is one of the special labels ("ksk", "zsk", or "all") then a
+set of key paths will be returned.
+A I<keytype> of "ksk" will return paths to all KSK keys for the zone,
+a I<keytype> of "zsk" will return paths to all ZSK keys for the zone,
+and a I<keytype> of "all" will return paths to all keys for the zone,
+
+If the given key type is not defined in the given zone's zone I<keyrec>
+or if the key type is not recognized, then a null set is returned.
 
 =item I<keyrec_names()>
 
@@ -2585,6 +2790,12 @@ following:  "kskcur", "kskpub", "kskrev", "kskobs", "zskcur", "zskpub",
 
 It returns 1 if the call is successful; 0 if it is not.
 
+=item I<keyrec_signset_prefix(zone_name)>
+
+I<keyrec_signset_prefix()> returns the signing set prefix formed by
+concatenating the zone name and $DEFSETPREFIX.  This prefix should be
+followed by a numeric index.
+
 =item I<keyrec_signset_addkey(signing_set_name,key_list)>
 
 I<keyrec_signset_addkey()> adds the keys listed in I<key_list> to the signing
@@ -2617,6 +2828,31 @@ It returns 1 if the call is successful; 0 if it is not.
 I<keyrec_signsets()> returns the names of the signing sets in the I<keyrec>
 file.  These names are returned in an array.
 
+=item I<keyrec_signset_keys(keyrec_name,signset_type)>
+
+I<keyrec_signset_keys()> returns the names of the keys that are members of
+a given signing set in the I<keyrec> file.  The keys are returned in a
+space-separated string.
+
+There are two ways of calling I<keyrec_signset_keys()>.   The first method
+specifies a zone I<keyrec> name and a signing set type.  The signing set name
+is found by referencing the set field in the zone I<keyrec>, then the I<keys>
+field of that signing set is returned.
+
+The second method specifies the signing set directly, and its I<keys> field
+is returned.
+
+Valid signing set types are: 
+
+    kskcur        kskpub        kskrev        kskobs
+    zskcur        zskpub        zsknew        zskobs
+
+The following errors are recognized, resulting in an undefined return:
+
+    keyrec_name is not a defined keyrec
+    signset_type is an invalid signing set type
+    the signing set keyrec is not a set keyrec
+
 =back
 
 =head1 KEYREC INTERNAL INTERFACES
@@ -2678,12 +2914,12 @@ I<keyrec> file.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+Copyright 2005-2013 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
 
-Wayne Morrison, tewok@users.sourceforge.net
+Wayne Morrison, tewok@tislabs.com
 
 =head1 SEE ALSO
 
