@@ -1,5 +1,5 @@
 #
-# Copyright 2005-2011 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2005-2013 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details
 #
 # DNSSEC Tools
@@ -96,8 +96,8 @@ use Net::DNS::SEC::Tools::conf;
 use Net::DNS::SEC::Tools::defaults;
 use Net::DNS::SEC::Tools::rolllog;
 
-our $VERSION = "1.9";
-our $MODULE_VERSION = "1.9.0";
+our $VERSION = "2.0";
+our $MODULE_VERSION = "2.0.0";
 
 our @ISA = qw(Exporter);
 
@@ -135,37 +135,49 @@ our @EXPORT = qw(
 			 ROLLCMD_LOGLEVEL
 			 ROLLCMD_LOGMSG
 			 ROLLCMD_LOGTZ
+			 ROLLCMD_MERGERRFS
 			 ROLLCMD_PHASEMSG
 			 ROLLCMD_ROLLALL
+			 ROLLCMD_ROLLALLKSKS
 			 ROLLCMD_ROLLALLZSKS
 			 ROLLCMD_ROLLKSK
 			 ROLLCMD_ROLLREC
 			 ROLLCMD_ROLLZONE
 			 ROLLCMD_ROLLZSK
 			 ROLLCMD_RUNQUEUE
+			 ROLLCMD_QUEUELIST
+			 ROLLCMD_QUEUESTATUS
 			 ROLLCMD_SHUTDOWN
+			 ROLLCMD_SIGNZONE
+			 ROLLCMD_SIGNZONES
 			 ROLLCMD_SKIPALL
 			 ROLLCMD_SKIPZONE
 			 ROLLCMD_SLEEPTIME
+			 ROLLCMD_SPLITRRF
 			 ROLLCMD_STATUS
 			 ROLLCMD_ZONELOG
+			 ROLLCMD_ZONEGROUP
 			 ROLLCMD_ZONESTATUS
 			 ROLLCMD_ZSARGS
 
-			 ROLLCMD_RC_OKAY
-			 ROLLCMD_RC_BADLEVEL
+			 ROLLMGR_GROUP
+
+			 ROLLCMD_RC_BADEVENT
 			 ROLLCMD_RC_BADFILE
-			 ROLLCMD_RC_BADSLEEP
+			 ROLLCMD_RC_BADLEVEL
 			 ROLLCMD_RC_BADROLLREC
+			 ROLLCMD_RC_BADSLEEP
 			 ROLLCMD_RC_BADTZ
-			 ROLLCMD_RC_DISPLAY
-			 ROLLCMD_RC_RRFOPEN
-			 ROLLCMD_RC_NOZONES
 			 ROLLCMD_RC_BADZONE
 			 ROLLCMD_RC_BADZONEDATA
+			 ROLLCMD_RC_BADZONEGROUP
+			 ROLLCMD_RC_DISPLAY
 			 ROLLCMD_RC_KSKROLL
-			 ROLLCMD_RC_ZSKROLL
 			 ROLLCMD_RC_NOARGS
+			 ROLLCMD_RC_NOZONES
+			 ROLLCMD_RC_OKAY
+			 ROLLCMD_RC_RRFOPEN
+			 ROLLCMD_RC_ZSKROLL
 
 			 CHANNEL_WAIT
 			 CHANNEL_CLOSE
@@ -185,6 +197,16 @@ my $EOL		= "\015\012";			# Net-standard end-of-line.
 my $CHANNEL_TYPE = PF_UNIX;			# Type of channel we're using.
 my $UNIXSOCK	= "/rollmgr.socket";		# Unix socket name.
 
+#
+# Maximum lengths of Unix socket names for various systems.  These are used
+# in rollmgr_channel().
+#
+my $FREEBSD_MAXSOCKNAME	= 103;
+my $MACOSX_MAXSOCKNAME	= 103;
+my $LINUX_MAXSOCKNAME	= 107;
+my $UNKNOWN_MAXSOCKNAME = $MACOSX_MAXSOCKNAME;		# Use the shortest.
+my $maxsockname		= $UNKNOWN_MAXSOCKNAME;
+
 #
 # The CHANNEL_ entities are used for specifying whether rollmgr_sendcmd()
 # should or should not wait for a response from rollerd.
@@ -213,75 +235,98 @@ my $ROLLCMD_RC_DISPLAY		= 10;
 my $ROLLCMD_RC_KSKROLL		= 11;
 my $ROLLCMD_RC_ZSKROLL		= 12;
 my $ROLLCMD_RC_NOARGS		= 13;
+my $ROLLCMD_RC_BADEVENT		= 14;
+my $ROLLCMD_RC_BADZONEGROUP	= 15;
 
 sub ROLLCMD_RC_OKAY		{ return($ROLLCMD_RC_OKAY);		};
-sub ROLLCMD_RC_BADLEVEL		{ return($ROLLCMD_RC_BADLEVEL);		};
+sub ROLLCMD_RC_BADEVENT		{ return($ROLLCMD_RC_BADEVENT);		};
 sub ROLLCMD_RC_BADFILE		{ return($ROLLCMD_RC_BADFILE);		};
-sub ROLLCMD_RC_BADSLEEP		{ return($ROLLCMD_RC_BADSLEEP);		};
+sub ROLLCMD_RC_BADLEVEL		{ return($ROLLCMD_RC_BADLEVEL);		};
 sub ROLLCMD_RC_BADROLLREC	{ return($ROLLCMD_RC_BADROLLREC);	};
+sub ROLLCMD_RC_BADSLEEP		{ return($ROLLCMD_RC_BADSLEEP);		};
 sub ROLLCMD_RC_BADTZ		{ return($ROLLCMD_RC_BADTZ);		};
-sub ROLLCMD_RC_RRFOPEN		{ return($ROLLCMD_RC_RRFOPEN);		};
-sub ROLLCMD_RC_NOZONES		{ return($ROLLCMD_RC_NOZONES);		};
 sub ROLLCMD_RC_BADZONE		{ return($ROLLCMD_RC_BADZONE);		};
 sub ROLLCMD_RC_BADZONEDATA	{ return($ROLLCMD_RC_BADZONEDATA);	};
+sub ROLLCMD_RC_BADZONEGROUP	{ return($ROLLCMD_RC_BADZONEGROUP);	};
 sub ROLLCMD_RC_DISPLAY		{ return($ROLLCMD_RC_DISPLAY);		};
 sub ROLLCMD_RC_KSKROLL		{ return($ROLLCMD_RC_KSKROLL);		};
-sub ROLLCMD_RC_ZSKROLL		{ return($ROLLCMD_RC_ZSKROLL);		};
 sub ROLLCMD_RC_NOARGS		{ return($ROLLCMD_RC_NOARGS);		};
+sub ROLLCMD_RC_NOZONES		{ return($ROLLCMD_RC_NOZONES);		};
+sub ROLLCMD_RC_RRFOPEN		{ return($ROLLCMD_RC_RRFOPEN);		};
+sub ROLLCMD_RC_ZSKROLL		{ return($ROLLCMD_RC_ZSKROLL);		};
 
 #
 # The remaining ROLLCMD_ entities are the rollmgr_sendcmd() commands
 # recognized by rollerd.  %roll_commands is a hash table of valid commands.
 #
-my $ROLLCMD_DISPLAY	= "rollcmd_display";
-my $ROLLCMD_DSPUB	= "rollcmd_dspub";
-my $ROLLCMD_DSPUBALL	= "rollcmd_dspuball";
-my $ROLLCMD_GETSTATUS	= "rollcmd_getstatus";
-my $ROLLCMD_LOGFILE	= "rollcmd_logfile";
-my $ROLLCMD_LOGLEVEL	= "rollcmd_loglevel";
-my $ROLLCMD_LOGMSG	= "rollcmd_logmsg";
-my $ROLLCMD_LOGTZ	= "rollcmd_logtz";
-my $ROLLCMD_PHASEMSG	= "rollcmd_phasemsg";
-my $ROLLCMD_ROLLALL	= "rollcmd_rollall";
-my $ROLLCMD_ROLLALLZSKS	= "rollcmd_rollallzsks";
-my $ROLLCMD_ROLLKSK	= "rollcmd_rollksk";
-my $ROLLCMD_ROLLREC	= "rollcmd_rollrec";
-my $ROLLCMD_ROLLZONE	= "rollcmd_rollzone";
-my $ROLLCMD_ROLLZSK	= "rollcmd_rollzsk";
-my $ROLLCMD_RUNQUEUE	= "rollcmd_runqueue";
-my $ROLLCMD_SHUTDOWN	= "rollcmd_shutdown";
-my $ROLLCMD_SKIPALL	= "rollcmd_skipall";
-my $ROLLCMD_SKIPZONE	= "rollcmd_skipzone";
-my $ROLLCMD_SLEEPTIME	= "rollcmd_sleeptime";
-my $ROLLCMD_STATUS	= "rollcmd_status";
-my $ROLLCMD_ZONELOG	= "rollcmd_zonelog";
-my $ROLLCMD_ZONESTATUS	= "rollcmd_zonestatus";
-my $ROLLCMD_ZSARGS	= "rollcmd_zsargs";
+my $ROLLCMD_DISPLAY	 = "rollcmd_display";
+my $ROLLCMD_DSPUB	 = "rollcmd_dspub";
+my $ROLLCMD_DSPUBALL	 = "rollcmd_dspuball";
+my $ROLLCMD_GETSTATUS	 = "rollcmd_getstatus";
+my $ROLLCMD_LOGFILE	 = "rollcmd_logfile";
+my $ROLLCMD_LOGLEVEL	 = "rollcmd_loglevel";
+my $ROLLCMD_LOGMSG	 = "rollcmd_logmsg";
+my $ROLLCMD_LOGTZ	 = "rollcmd_logtz";
+my $ROLLCMD_MERGERRFS	 = "rollcmd_mergerrfs";
+my $ROLLCMD_PHASEMSG	 = "rollcmd_phasemsg";
+my $ROLLCMD_ROLLALL	 = "rollcmd_rollall";
+my $ROLLCMD_ROLLALLKSKS	 = "rollcmd_rollallksks";
+my $ROLLCMD_ROLLALLZSKS	 = "rollcmd_rollallzsks";
+my $ROLLCMD_ROLLKSK	 = "rollcmd_rollksk";
+my $ROLLCMD_ROLLREC	 = "rollcmd_rollrec";
+my $ROLLCMD_ROLLZONE	 = "rollcmd_rollzone";
+my $ROLLCMD_ROLLZSK	 = "rollcmd_rollzsk";
+my $ROLLCMD_RUNQUEUE	 = "rollcmd_runqueue";
+my $ROLLCMD_QUEUELIST	 = "rollcmd_queuelist";
+my $ROLLCMD_QUEUESTATUS	 = "rollcmd_queuestatus";
+my $ROLLCMD_SHUTDOWN	 = "rollcmd_shutdown";
+my $ROLLCMD_SIGNZONE	 = "rollcmd_signzone";
+my $ROLLCMD_SIGNZONES	 = "rollcmd_signzones";
+my $ROLLCMD_SKIPALL	 = "rollcmd_skipall";
+my $ROLLCMD_SKIPZONE	 = "rollcmd_skipzone";
+my $ROLLCMD_SLEEPTIME	 = "rollcmd_sleeptime";
+my $ROLLCMD_SPLITRRF	 = "rollcmd_splitrrf";
+my $ROLLCMD_STATUS	 = "rollcmd_status";
+my $ROLLCMD_ZONEGROUP	 = "rollcmd_zonegroup";
+my $ROLLCMD_ZONELOG	 = "rollcmd_zonelog";
+my $ROLLCMD_ZONESTATUS	 = "rollcmd_zonestatus";
+my $ROLLCMD_ZSARGS	 = "rollcmd_zsargs";
 
-sub ROLLCMD_DISPLAY		{ return($ROLLCMD_DISPLAY);	};
-sub ROLLCMD_DSPUB		{ return($ROLLCMD_DSPUB);	};
-sub ROLLCMD_DSPUBALL		{ return($ROLLCMD_DSPUBALL);	};
-sub ROLLCMD_GETSTATUS		{ return($ROLLCMD_GETSTATUS);	};
-sub ROLLCMD_LOGFILE		{ return($ROLLCMD_LOGFILE);	};
-sub ROLLCMD_LOGLEVEL		{ return($ROLLCMD_LOGLEVEL);	};
-sub ROLLCMD_LOGMSG		{ return($ROLLCMD_LOGMSG);	};
-sub ROLLCMD_LOGTZ		{ return($ROLLCMD_LOGTZ);	};
-sub ROLLCMD_PHASEMSG		{ return($ROLLCMD_PHASEMSG);	};
-sub ROLLCMD_ROLLALL		{ return($ROLLCMD_ROLLALL);	};
-sub ROLLCMD_ROLLALLZSKS		{ return($ROLLCMD_ROLLALLZSKS);	};
-sub ROLLCMD_ROLLKSK		{ return($ROLLCMD_ROLLKSK);	};
-sub ROLLCMD_ROLLREC		{ return($ROLLCMD_ROLLREC);	};
-sub ROLLCMD_ROLLZONE		{ return($ROLLCMD_ROLLZONE);	};
-sub ROLLCMD_ROLLZSK		{ return($ROLLCMD_ROLLZSK);	};
-sub ROLLCMD_RUNQUEUE		{ return($ROLLCMD_RUNQUEUE);	};
-sub ROLLCMD_SHUTDOWN		{ return($ROLLCMD_SHUTDOWN);	};
-sub ROLLCMD_SKIPALL		{ return($ROLLCMD_SKIPALL);	};
-sub ROLLCMD_SKIPZONE		{ return($ROLLCMD_SKIPZONE);	};
-sub ROLLCMD_SLEEPTIME		{ return($ROLLCMD_SLEEPTIME);	};
-sub ROLLCMD_STATUS		{ return($ROLLCMD_STATUS);	};
-sub ROLLCMD_ZONELOG		{ return($ROLLCMD_ZONELOG);	};
-sub ROLLCMD_ZONESTATUS		{ return($ROLLCMD_ZONESTATUS);	};
-sub ROLLCMD_ZSARGS		{ return($ROLLCMD_ZSARGS);	};
+sub ROLLCMD_DISPLAY		{ return($ROLLCMD_DISPLAY);	 };
+sub ROLLCMD_DSPUB		{ return($ROLLCMD_DSPUB);	 };
+sub ROLLCMD_DSPUBALL		{ return($ROLLCMD_DSPUBALL);	 };
+sub ROLLCMD_GETSTATUS		{ return($ROLLCMD_GETSTATUS);	 };
+sub ROLLCMD_LOGFILE		{ return($ROLLCMD_LOGFILE);	 };
+sub ROLLCMD_LOGLEVEL		{ return($ROLLCMD_LOGLEVEL);	 };
+sub ROLLCMD_LOGMSG		{ return($ROLLCMD_LOGMSG);	 };
+sub ROLLCMD_LOGTZ		{ return($ROLLCMD_LOGTZ);	 };
+sub ROLLCMD_MERGERRFS		{ return($ROLLCMD_MERGERRFS);	 };
+sub ROLLCMD_PHASEMSG		{ return($ROLLCMD_PHASEMSG);	 };
+sub ROLLCMD_ROLLALL		{ return($ROLLCMD_ROLLALL);	 };
+sub ROLLCMD_ROLLALLKSKS		{ return($ROLLCMD_ROLLALLKSKS);	 };
+sub ROLLCMD_ROLLALLZSKS		{ return($ROLLCMD_ROLLALLZSKS);	 };
+sub ROLLCMD_ROLLKSK		{ return($ROLLCMD_ROLLKSK);	 };
+sub ROLLCMD_ROLLREC		{ return($ROLLCMD_ROLLREC);	 };
+sub ROLLCMD_ROLLZONE		{ return($ROLLCMD_ROLLZONE);	 };
+sub ROLLCMD_ROLLZSK		{ return($ROLLCMD_ROLLZSK);	 };
+sub ROLLCMD_RUNQUEUE		{ return($ROLLCMD_RUNQUEUE);	 };
+sub ROLLCMD_QUEUELIST		{ return($ROLLCMD_QUEUELIST);	 };
+sub ROLLCMD_QUEUESTATUS		{ return($ROLLCMD_QUEUESTATUS);	 };
+sub ROLLCMD_SHUTDOWN		{ return($ROLLCMD_SHUTDOWN);	 };
+sub ROLLCMD_SIGNZONE		{ return($ROLLCMD_SIGNZONE);	 };
+sub ROLLCMD_SIGNZONES		{ return($ROLLCMD_SIGNZONES);	 };
+sub ROLLCMD_SKIPALL		{ return($ROLLCMD_SKIPALL);	 };
+sub ROLLCMD_SKIPZONE		{ return($ROLLCMD_SKIPZONE);	 };
+sub ROLLCMD_SLEEPTIME		{ return($ROLLCMD_SLEEPTIME);	 };
+sub ROLLCMD_SPLITRRF		{ return($ROLLCMD_SPLITRRF);	 };
+sub ROLLCMD_STATUS		{ return($ROLLCMD_STATUS);	 };
+sub ROLLCMD_ZONEGROUP		{ return($ROLLCMD_ZONEGROUP);	 };
+sub ROLLCMD_ZONELOG		{ return($ROLLCMD_ZONELOG);	 };
+sub ROLLCMD_ZONESTATUS		{ return($ROLLCMD_ZONESTATUS);	 };
+sub ROLLCMD_ZSARGS		{ return($ROLLCMD_ZSARGS);	 };
+
+my $ROLLMGR_GROUP	= "g-";
+sub ROLLMGR_GROUP		{ return($ROLLMGR_GROUP);	};
 
 my %roll_commands =
 (
@@ -293,20 +338,28 @@ my %roll_commands =
 	rollcmd_loglevel	=> 1,
 	rollcmd_logmsg		=> 1,
 	rollcmd_logtz		=> 1,
+	rollcmd_mergerrfs	=> 1,
 	rollcmd_nodisplay	=> 1,
 	rollcmd_phasemsg	=> 1,
 	rollcmd_rollall		=> 1,
+	rollcmd_rollallksks	=> 1,
 	rollcmd_rollallzsks	=> 1,
 	rollcmd_rollksk		=> 1,
 	rollcmd_rollrec		=> 1,
 	rollcmd_rollzone	=> 1,
 	rollcmd_rollzsk		=> 1,
 	rollcmd_runqueue	=> 1,
+	rollcmd_queuelist	=> 1,
+	rollcmd_queuestatus	=> 1,
 	rollcmd_shutdown	=> 1,
+	rollcmd_signzones	=> 1,
+	rollcmd_signzone	=> 1,
 	rollcmd_skipall		=> 1,
 	rollcmd_skipzone	=> 1,
 	rollcmd_sleeptime	=> 1,
+	rollcmd_splitrrf	=> 1,
 	rollcmd_status		=> 1,
+	rollcmd_zonegroup	=> 1,
 	rollcmd_zonelog		=> 1,
 	rollcmd_zonestatus	=> 1,
 	rollcmd_zsargs		=> 1,
@@ -368,7 +421,7 @@ my %switch_unix =
 	$CMDINT	   =>	\&unix_cmdint,
 	$DROPID	   =>	\&unix_dropid,
 	$GETDIR	   =>	\&unix_dir,
-	$GETID	   =>	\&unix_getid,
+	$GETID	   =>	\&unix_getpid,
 	$HALT	   =>	\&unix_halt,
 	$IDFILE	   =>	\&unix_idfile,
 	$SETIDFILE =>	\&unix_set_idfile,
@@ -383,7 +436,7 @@ my %switch_sysv =
 	$CMDINT	   =>	\&unix_cmdint,
 	$DROPID	   =>	\&unix_psef_dropid,
 	$GETDIR	   =>	\&unix_dir,
-	$GETID	   =>	\&unix_getid,
+	$GETID	   =>	\&unix_getpid,
 	$HALT	   =>	\&unix_halt,
 	$IDFILE	   =>	\&unix_idfile,
 	$SETIDFILE =>	\&unix_set_idfile,
@@ -519,6 +572,23 @@ sub rollmgr_prepdep
 		$osclass = "sysv";
 	}
 
+	#
+	# Set the maximum socket-name length depending on which operating
+	# system class we're running on.
+	#
+	if($osname eq "darwin")
+	{
+		$maxsockname = $MACOSX_MAXSOCKNAME;
+	}
+	elsif($osname eq "freebsd")
+	{
+		$maxsockname = $FREEBSD_MAXSOCKNAME;
+	}
+	elsif($osname eq "linux")
+	{
+		$maxsockname = $LINUX_MAXSOCKNAME;
+	}
+
 	#
 	# Get the appropriate switch table for this O/S class and save
 	# it for later reference.
@@ -535,7 +605,7 @@ sub rollmgr_prepdep
 #
 sub rollmgr_dir
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 	my $func;				# Actual function.
 
 # print "rollmgr_dir\n";
@@ -552,7 +622,7 @@ sub rollmgr_dir
 #
 sub rollmgr_dropid
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 	my $func;				# Actual function.
 
 # print "rollmgr_dropid\n";
@@ -569,7 +639,7 @@ sub rollmgr_dropid
 #
 sub rollmgr_getid
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 	my $func;				# Actual function.
 
 # print "rollmgr_getid\n";
@@ -586,7 +656,7 @@ sub rollmgr_getid
 #
 sub rollmgr_halt
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 	my $func;				# Actual function.
 
 # print "rollmgr_halt\n";
@@ -603,7 +673,7 @@ sub rollmgr_halt
 #
 sub rollmgr_idfile
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 	my $func;				# Actual function.
 
 # print "rollmgr_idfile\n";
@@ -620,7 +690,7 @@ sub rollmgr_idfile
 #
 sub rollmgr_set_idfile
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 	my $func;				# Actual function.
 
 # print "rollmgr_set_idfile\n";
@@ -636,7 +706,7 @@ sub rollmgr_set_idfile
 #
 sub rollmgr_loadzone
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 	my $func;				# Actual function.
 
 # print "rollmgr_loadzone\n";
@@ -680,7 +750,7 @@ sub rollmgr_phasemsg
 #
 sub rollmgr_cmdint
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 	my $func;				# Actual function.
 
 # print "rollmgr_cmdint\n";
@@ -697,7 +767,7 @@ sub rollmgr_cmdint
 #
 sub rollmgr_rmid
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 	my $func;				# Actual function.
 
 # print "rollmgr_rmid\n";
@@ -713,7 +783,7 @@ sub rollmgr_rmid
 #
 sub rollmgr_running
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 	my $func;				# Actual function.
 
 # print "rollmgr_running\n";
@@ -730,7 +800,7 @@ sub rollmgr_running
 #
 sub rollmgr_saveid
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 	my $func;				# Actual function.
 
 # print "rollmgr_saveid\n";
@@ -760,7 +830,7 @@ sub rollmgr_saveid
 #
 sub uninit_dir
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 
 # print "uninit_dir\n";
 
@@ -775,7 +845,7 @@ sub uninit_dir
 #
 sub uninit_dropid
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 
 # print "uninit_dropid\n";
 
@@ -790,7 +860,7 @@ sub uninit_dropid
 #
 sub uninit_getid
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 
 # print "uninit_getid\n";
 
@@ -805,7 +875,7 @@ sub uninit_getid
 #
 sub uninit_halt
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 
 # print "uninit_halt\n";
 
@@ -820,7 +890,7 @@ sub uninit_halt
 #
 sub uninit_idfile
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 
 # print "uninit_idfile\n";
 
@@ -835,7 +905,7 @@ sub uninit_idfile
 #
 sub uninit_set_idfile
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 
 # print "uninit_set_idfile\n";
 
@@ -850,7 +920,7 @@ sub uninit_set_idfile
 #
 sub uninit_loadzone
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 
 # print "uninit_loadzone\n";
 
@@ -865,7 +935,7 @@ sub uninit_loadzone
 #
 sub uninit_cmdint
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 
 # print "uninit_cmdint\n";
 
@@ -880,7 +950,7 @@ sub uninit_cmdint
 #
 sub uninit_rmid
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 
 # print "uninit_rmid\n";
 
@@ -895,7 +965,7 @@ sub uninit_rmid
 #
 sub uninit_running
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 
 # print "uninit_running\n";
 
@@ -910,7 +980,7 @@ sub uninit_running
 #
 sub uninit_saveid
 {
-	my @args = shift;			# Routine arguments.
+	my @args = @_;				# Routine arguments.
 
 # print "uninit_saveid\n";
 
@@ -1500,8 +1570,8 @@ sub rollmgr_channel
 		if($server)
 		{
 			setsockopt(SOCK,SOL_SOCKET,SO_REUSEADDR,pack("l",1));
-			bind(SOCK,sockaddr_in($CMDPORT,$ADDR)) || return(0);
-			listen(SOCK,SOMAXCONN) || return(0);
+			bind(SOCK,sockaddr_in($CMDPORT,$ADDR)) || return(-2);
+			listen(SOCK,SOMAXCONN) || return(-4);
 		}
 		else
 		{
@@ -1524,6 +1594,16 @@ sub rollmgr_channel
 		$unixsock = makelocalstatedir("/dnssec-tools") . $UNIXSOCK;
 # print STDERR "rollmgr_channel:  unixsock - <$unixsock>\n";
 
+		#
+		# Ensure the socket name isn't too long.  This is a result
+		# of a hardcode maximum length for socket names.  This is
+		# in the kernel and isn't 
+		#
+		return(-5) if(length($unixsock) > $maxsockname);
+
+		#
+		# Create the socket.
+		#
 		$sockdata = sockaddr_un($unixsock);
 
 		#
@@ -1854,7 +1934,27 @@ sub rollmgr_verifycmd
 	$hval = $roll_commands{$cmd};
 # print "rollmgr_verifycmd:  <$cmd>\t\t<$hval>\n";
 
-	return(0) if(!defined($hval));
+	#
+	# If the command is undefined, we'll check if it's a group
+	# command.  If so, we'll strip the group indicator and try again.
+	#
+	if(!defined($hval))
+	{
+		my $gstr = ROLLMGR_GROUP;	# Group command indicator.
+		if($cmd =~ /^$gstr/)
+		{
+			$cmd =~ s/^$gstr//;
+			return(rollmgr_verifycmd($cmd));
+		}
+		else
+		{
+			return(0);
+		}
+	}
+
+	#
+	# Success!
+	#
 	return(1);
 }
 
@@ -2069,6 +2169,16 @@ If I<serverflag> is false, then the client's side of the channel is created.
 Currently, the connection may only be made to the localhost.  This may be
 changed to allow remote connections, if this is found to be needed.
 
+Return Values:
+
+      1 - Communications channel successfully established.
+      0 - Unable to connect to the server.
+     -1 - Unable to create a Unix socket.
+     -2 - Unable to bind to the Unix socket. (server only)
+     -3 - Unable to change the permissions on the Unix socket. (server only)
+     -4 - Unable to listen on the Unix socket. (server only)
+     -5 - The socket name was longer than allowed for a Unix socket.
+
 =item I<rollmgr_queuecmd(cmdname, value)>
 
 This interface internally remembers a command and it's optional value
@@ -2102,14 +2212,24 @@ The available commands and their required data are:
    ROLLCMD_DSPUB	zone-name	a DS record has been
 					published
    ROLLCMD_DSPUBALL	none		DS records published for all
-					zones in KSK rollover phase 6
+					zones in KSK rollover phase 5
    ROLLCMD_GETSTATUS	none		currently unused
    ROLLCMD_LOGFILE	log filename	change the log file
    ROLLCMD_LOGLEVEL	log level	set a new logging level
    ROLLCMD_LOGMSG	log message	add a message to the log
    ROLLCMD_LOGTZ	timezone	set timezone for log messages
+   ROLLCMD_MERGERRFS	rollrec files	merge rollrec files with the
+					current rollrec file
    ROLLCMD_PHASEMSG	long/short	set long or short phase
 					messages
+   ROLLCMD_QUEUELIST	none		returns the names and next
+					event time of zones in the
+					"soon queue
+					(experimental)
+   ROLLCMD_QUEUESTATUS	none		returns information about
+  					the state of soon-queue
+  					processing
+					(experimental)
    ROLLCMD_ROLLALL	none		resume rollover for all
 					suspended zones
    ROLLCMD_ROLLALLZSKS	none		force all zones to start
@@ -2124,11 +2244,18 @@ The available commands and their required data are:
    ROLLCMD_RUNQUEUE	none		rollerd runs through
 					its queue
    ROLLCMD_SHUTDOWN	none		stop rollerd
+   ROLLCMD_SIGNZONE	zone		sign a zone (no rollover)
+   ROLLCMD_SIGNZONEs	all or active	sign all or active zones
    ROLLCMD_SKIPALL	none		suspend all rollovers
    ROLLCMD_SKIPZONE	zone name	suspend rollover for a
 					rolling zone
    ROLLCMD_SLEEPTIME	seconds-count	set rollerd's sleep time
+   ROLLCMD_SPLITRRF	rollrec-name,	move a set of zones from the
+			zone names	current rollrec file into a
+					new rollrec file
    ROLLCMD_STATUS	none		get status of rollerd
+   ROLLCMD_ZONEGROUP	zonegroup name	get info on all zonegroups
+					or a particular zonegroup
    ROLLCMD_ZONELOG	zone name	set the logging level for
 			logging level	a particular zone
    ROLLCMD_ZONESTATUS	none		get status of the zones
@@ -2183,12 +2310,12 @@ the file prior to opening it, but we can't do so without it being open.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+Copyright 2005-2013 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
 
-Wayne Morrison, tewok@users.sourceforge.net
+Wayne Morrison, tewok@tislabs.com
 
 =head1 SEE ALSO
 
