@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# Copyright 2005-2011 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2005-2013 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details
 #
 # DNSSEC Tools
@@ -8,7 +8,7 @@
 # timetrans
 #
 #       This module contains an interface to convert an integer seconds count
-#	into the equivalent number of weeks, days, hours, and minutes.
+#	into the equivalent number of days, hours, and minutes.
 #
 
 package Net::DNS::SEC::Tools::timetrans;
@@ -19,10 +19,10 @@ require Exporter;
 
 our @ISA = qw(Exporter);
 
-our @EXPORT = qw(timetrans fuzzytimetrans);
+our @EXPORT = qw(timetrans fuzzytimetrans timetrans_weeks);
 
-our $VERSION = "1.9";
-our $MODULE_VERSION = "1.9.0";
+our $VERSION = "2.0";
+our $MODULE_VERSION = "2.0.0";
 
 
 #######################################################################
@@ -39,9 +39,126 @@ my $WEEK   = (7  * $DAY);
 # Routine:	timetrans()
 #
 # Purpose:	This routine converts an integer seconds count into the
-#		equivalent number of weeks, days, hours, and minutes.
+#		equivalent number of days, hours, and minutes.
 #
 sub timetrans
+{
+	my $seconds = shift;			# The seconds to be translated.
+	my $minutes;				# Minutes in seconds.
+	my $hours;				# Hours in seconds.
+	my $days;				# Days in seconds.
+
+	my $tstr;				# Time string.
+	my $sstr;				# Seconds string.
+	my $mstr;				# Minutes string.
+	my $hstr;				# Hours string.
+	my $dstr;				# Days string.
+
+	#
+	# Ensure we were given a valid seconds count.
+	#
+	return("") if($seconds < 0);
+
+	#
+	# Check for zero seconds.
+	#
+#	return("0 seconds") if($seconds == 0);
+	return("") if($seconds == 0);
+
+	#
+	# Handle the less-than-a-minute case.
+	#
+	if($seconds < $MINUTE)
+	{
+		$tstr = "$seconds second";
+		$tstr = $tstr . "s" if($seconds != 1);
+
+		return("") if($seconds == 0);
+		return($tstr);
+	}
+
+	#
+	# Handle the less-than-an-hour case.
+	#
+	if($seconds < $HOUR)
+	{
+		$minutes = $seconds / $MINUTE;
+		$seconds = $seconds % $MINUTE;
+
+		$sstr = timetrans($seconds);
+		if($sstr ne "")
+		{
+			$tstr = sprintf("%d minutes, $sstr",$minutes);
+		}
+		else
+		{
+			$tstr = sprintf("%d minutes",$minutes);
+		}
+
+		$tstr =~ s/minutes/minute/ if($tstr =~ /^1 /);
+
+		return("") if($minutes == 0);
+		return($tstr);
+	}
+
+	#
+	# Handle the less-than-a-day case.
+	#
+	if($seconds < $DAY)
+	{
+		$hours = $seconds / $HOUR;
+		$minutes = $seconds % $HOUR;
+
+		$mstr = timetrans($minutes);
+		if($mstr ne "")
+		{
+			$tstr = sprintf("%d hours, $mstr",$hours);
+		}
+		else
+		{
+			$tstr = sprintf("%d hours",$hours);
+		}
+
+		$tstr =~ s/hours/hour/ if($tstr =~ /^1 /);
+
+		return("") if($hours == 0);
+		return($tstr);
+	}
+
+	#
+	# The rest of the cases all fall into days.
+	#
+	$days  = $seconds / $DAY;
+	$hours	= $seconds % $DAY;
+
+	$hstr = timetrans($hours);
+	if($hstr ne "")
+	{
+		$tstr = sprintf("%d days, $hstr",$days);
+	}
+	else
+	{
+		$tstr = sprintf("%d days",$days);
+	}
+
+	$tstr =~ s/days/day/ if($tstr =~ /^1 /);
+
+	return($tstr);
+}
+
+#---------------------------------------------------------------------------
+#
+# Routine:	timetrans_weeks()
+#
+# Purpose:	This routine converts an integer seconds count into the
+#		equivalent number of weeks, days, hours, and minutes.
+#
+#		WARNING:  This routine is unsupported and will most likely
+#			  disappear before long.  If you really need it and
+#			  don't want the routine to go away, bring this up
+#			  on the dnssec-tools user list.
+#
+sub timetrans_weeks
 {
 	my $seconds = shift;			# The seconds to be translated.
 	my $minutes;				# Minutes in seconds.
@@ -63,7 +180,8 @@ sub timetrans
 	#
 	# Check for zero seconds.
 	#
-	return("0 seconds") if($seconds == 0);
+#	return("0 seconds") if($seconds == 0);
+	return("") if($seconds == 0);
 
 	#
 	# Handle the less-than-a-minute case.
@@ -260,11 +378,11 @@ Net::DNS::SEC::Tools::timetrans - Convert an integer seconds count into text uni
 
 =head1 DESCRIPTION
 
-The I<timetrans>() interface in B<Net::DNS::SEC::Tools::timetrans>
-converts an integer seconds count into the equivalent number of weeks,
-days, hours, and minutes.  The time converted is a relative time, B<not>
-an absolute time.  The returned time is given in terms of weeks, days, hours,
-minutes, and seconds, as required to express the seconds count appropriately.
+The I<timetrans>() interface in B<Net::DNS::SEC::Tools::timetrans> converts an
+integer seconds count into the equivalent number of days, hours, and minutes.
+The time converted is a relative time, B<not> an absolute time.  The returned
+time is given in terms of days, hours, minutes, and seconds, as required to
+express the seconds count appropriately.
 
 The I<fuzzytimetrans>() interface converts an integer seconds count into the
 equivalent number of weeks B<or> days B<or> hours B<or> minutes.  The unit
@@ -279,9 +397,9 @@ below.
 =head2 B<timetrans()>
 
 This routine converts an integer seconds count into the equivalent number of
-weeks, days, hours, and minutes.  This converted seconds count is returned
-as a text string.  The seconds count must be greater than zero or an error
-will be returned.
+days, hours, and minutes.  This converted seconds count is returned as a text
+string.  The seconds count must be greater than zero or an error will be
+returned.
 
 Return Values:
 
@@ -318,9 +436,9 @@ I<timetrans(86400)> returns 1 day
 
 I<timetrans(86488)> returns 1 day, 28 seconds
 
-I<timetrans(715000)> returns 1 week, 1 day, 6 hours, 36 minutes, 40 second
+I<timetrans(715000)> returns 8 days, 6 hours, 36 minutes, 40 second
 
-I<timetrans(720000)> returns 1 week, 1 day, 8 hours
+I<timetrans(720000)> returns 8 days, 8 hours
 
 I<fuzzytimetrans(400)> returns 6.7 minutes
 
@@ -338,12 +456,12 @@ I<fuzzytimetrans(720000)> returns 1.2 weeks
 
 =head1 COPYRIGHT
 
-Copyright 2004-2011 SPARTA, Inc.  All rights reserved.
+Copyright 2004-2013 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
 
-Wayne Morrison, tewok@users.sourceforge.net
+Wayne Morrison, tewok@tislabs.com
 
 =head1 SEE ALSO
 
