@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# Copyright 2005-2011 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2005-2013 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details.
 #
 # DNSSEC-Tools
@@ -27,12 +27,13 @@ use strict;
 use Net::DNS::SEC::Tools::conf;
 use Net::DNS::SEC::Tools::keyrec;
 
-our $VERSION = "1.9";
-our $MODULE_VERSION = "1.9.0";
+our $VERSION = "2.0";
+our $MODULE_VERSION = "2.0.0";
 
 our @ISA = qw(Exporter);
 
 our @EXPORT = qw(
+			opts_cmdline
 			opts_cmdopts
 			opts_createkrf
 			opts_drop
@@ -90,6 +91,7 @@ my @stdopts =
 		["ksklength=i",		"Length of KSK"],
 		["ksklife=i",		"Life-span of KSK"],
 		["revperiod=i",		"Revocation period"],
+		["dsdir=s",		"Directory for dsset files"],
 		["ksdir=s",		"Directory for keyset files"],
 
 	'',
@@ -653,7 +655,8 @@ sub opts_int_zonecopy
 	#
 	# Copy the data from the Current KSK and Current ZSK keyrecs.
 	#
-	foreach my $ktype ('kskcur', 'zskcur')
+#	foreach my $ktype ('kskcur', 'zskcur')
+	foreach my $ktype ('kskcur', 'zskcur', 'kskpub', 'zskpub', 'zsknew')
 	{
 		my $setname;			# Name of the signing set.
 		my $setkeys;			# Keys in the signing set.
@@ -692,6 +695,63 @@ sub opts_int_zonecopy
 	return(%fields);
 }
 
+##############################################################################
+# Routine:	opts_cmdline()
+#
+# Purpose:	Parse a command line looking for the arguments in the standard
+#		set of options and the caller's set.  If the first argument is
+#		true, the program-wide @ARGV is restored after parsing.  If the
+#		caller provides other arguments, they're added as additional
+#		options.  The parsed options are returned to the caller.
+#
+sub opts_cmdline
+{
+	my $saveargv = shift;			# Flag for saving @ARGV.
+	my @csopts = @_;			# Command-specific options.
+
+	my @args;				# Copy of @ARGV.
+	my @opts;				# Copy of standard options.
+	my %parsedopts;				# Parsed options.
+
+	my $curgui = $gui;			# Saved $gui value.
+	my $curfc = $firstcall;			# Saved $firstcall value.
+
+	#
+	# Save the argument vector.
+	#
+	@args = @ARGV;
+
+	#
+	# Add the standard options to the caller's options.
+	#
+	@opts = @stdopts;
+	push @opts, @csopts if(@csopts > 0);
+
+	#
+	# Force some flags we need.
+	#
+	$gui = 0;
+	$firstcall = 0;
+
+	#
+	# Extract the command line options.
+	#
+	localgetoptions(\%parsedopts,@opts);
+
+	#
+	# Restore the forced flags.
+	#
+	$gui = $curgui;
+	$firstcall = $curfc;
+
+	#
+	# Restore the saved argument vector.
+	#
+	@ARGV = @args if($saveargv);
+
+	return(%parsedopts);
+}
+
 ##############################################################################
 #
 sub opts_int_cmdline
@@ -829,6 +889,8 @@ Net::DNS::SEC::Tools::tooloptions - DNSSEC-Tools option routines.
 
   @specopts = ("propagate+", "waittime=i");
 
+  %opts = opts_cmdline($restoreargv,@calleropts);
+
   $optsref = opts_cmdopts(@specopts);
   %options = %$optsref;
 
@@ -945,9 +1007,17 @@ will read each option source in turn, ending up with:
 
 =over 4
 
+=item I<opts_cmdline($restoreargv,@calleropts)>
+
+This routine parses a command line looking for the arguments in the standard
+set of options and an optional set of options specified by the caller.  If the
+first argument is true, the program-wide @ARGV is restored after parsing.  If
+the caller provides other arguments, they're added as additional options.  The
+parsed options are returned to the caller in a hash.
+
 =item I<opts_cmdopts(@csopts)>
 
-This I<opts_cmdopts()> call builds an option hash from the system configuration
+The I<opts_cmdopts()> call builds an option hash from the system configuration
 file, a I<keyrec>, and a set of command-specific options.  A reference to
 this option hash is returned to the caller.
 
@@ -970,8 +1040,8 @@ folded in as well, but the key's I<keyrec_> fields are excluded.  This
 call ensures that the named I<keyrec> is a zone I<keyrec>; if it isn't,
 I<undef> is returned.
 
-The I<keyrec> file is reading with I<keyrec_read()>.  To ensure it is
-properly read, I<keyrec_close()> is called first.
+The I<keyrec> file is read with I<keyrec_read()>.  To ensure it is properly
+read, I<keyrec_close()> is called first.
 
 The I<$keyrec_file> argument specifies a I<keyrec> file that will be
 consulted.  The I<keyrec> named by the I<$keyrec_name> argument will be
@@ -1047,12 +1117,12 @@ The old exit action is returned.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2011 SPARTA, Inc.  All rights reserved.
+Copyright 2005-2013 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
 
-Wayne Morrison, tewok@users.sourceforge.net
+Wayne Morrison, tewok@tislabs.com
 
 =head1 SEE ALSO
 
