@@ -12,6 +12,10 @@
 
 #define USE_QML 1
 
+extern "C" {
+void val_freeaddrinfo(addrinfo *foo) { }
+}
+
 int main(int argc, char *argv[])
 {
     QApplication app(argc, argv);
