@@ -11,16 +11,19 @@ class Filter : public QObject
 {
     Q_OBJECT
 public:
-    Filter();
+    Filter(QObject *parent = 0);
 
     virtual bool      matches(Node *node) = 0;
     virtual QString   name() = 0;
     virtual void      configWidgets(QHBoxLayout *hbox) { Q_UNUSED(hbox); }
 
-    void              filterHasChanged() { emit filterChanged(); qDebug() << "here: changing";}
+    void              filterHasChanged() { emit filterChanged(); }
+
+    static Filter    *getNewFilterFromMenu(QPoint where);
 
 signals:
     void              filterChanged();
+    void              filterAdded();
 };
 
 #endif // FILTER_H
