@@ -4,8 +4,8 @@
 #include <QtGui/QLineEdit>
 #include <qdebug.h>
 
-NameFilter::NameFilter(const QString &searchName)
-    : m_searchName(searchName.toLower())
+NameFilter::NameFilter(const QString &searchName, QObject *parent)
+    : Filter(parent), m_searchName(searchName.toLower())
 {
     setRegExp();
 }
@@ -14,6 +14,7 @@ void NameFilter::setSearchName(QString searchName)
 {
     m_searchName = searchName.toLower();
     setRegExp();
+    emit filterChanged();
 }
 
 QString NameFilter::searchName() const
@@ -29,7 +30,9 @@ void NameFilter::setRegExp()
 
 bool NameFilter::matches(Node *node)
 {
-    if (m_regexp.isEmpty() || m_regexp.indexIn(node->fqdn()) != -1)
+    if (m_regexp.isEmpty())
+        return false;
+    if (m_regexp.indexIn(node->fqdn()) != -1)
         return true;
     return false;
 }
@@ -37,6 +40,7 @@ bool NameFilter::matches(Node *node)
 void NameFilter::configWidgets(QHBoxLayout *hbox)
 {
     QLabel *filterLabel = new QLabel("Filter by RegExp:");
+    filterLabel->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);
     hbox->addWidget(filterLabel);
 
     QLineEdit *filterEditBox = new QLineEdit();
