@@ -1,14 +1,9 @@
 #include "NodeList.h"
 #include "DelayedDelete.h"
 
-#include "Filters/DNSSECStatusFilter.h"
-#include "Filters/NameFilter.h"
-#include "Filters/NotFilter.h"
-#include "Filters/TypeFilter.h"
+#include "filtersAndEffects.h"
 
-#include "Effects/SetAlphaEffect.h"
-#include "Effects/SetZValue.h"
-#include "Effects/MultiEffect.h"
+#include "graphwidget.h"
 
 #include <qdebug.h>
 
@@ -17,25 +12,40 @@
 NodeList::NodeList(GraphWidget *parent) :
     QObject(parent), m_graphWidget(parent), m_centerNode(0), m_nodes(), m_edges(),
     m_timer(this), m_maxNodes(0), m_accessCounter(0), m_accessDropOlderThan(0), m_selectedNode(0),
-    m_filtersAndEffects()
+    m_filtersAndEffects(), m_filterEditor(0)
 {
     connect(&m_timer, SIGNAL(timeout()), this, SLOT(limit()));
     m_timer.start(5000); /* clear things out every 5 seconds or so */
 }
 
+QString NodeList::removeTrailingDots(const QString &from) {
+    QString maybeToLong(from);
+    while (maybeToLong.endsWith("."))
+        maybeToLong.chop(1);
+    if (maybeToLong.length() == 0)
+        maybeToLong = "<root>";
+    return maybeToLong;
+}
+
 Node *NodeList::node(const QString &nodeName) {
-    if (! m_nodes.contains(nodeName)) {
-        return addNodes(nodeName);
+    QString maybeToLong = nodeName; //removeTrailingDots(nodeName);
+    if (maybeToLong == "")
+        maybeToLong = ROOT_NODE_NAME;
+
+    if (! m_nodes.contains(maybeToLong)) {
+        return addNodes(maybeToLong);
     }
 
-    return m_nodes[nodeName];
+    return m_nodes[maybeToLong];
 }
 
 Node *NodeList::addNodes(const QString &nodeName) {
     int count = 1;
     Node *returnNode = 0;
+    QString newNodeName = removeTrailingDots(nodeName);
 
-    QStringList nodeNameList = nodeName.split(".");
+
+    QStringList nodeNameList = newNodeName.split(".");
     QString completeString = QString("");
 
     QStringList::iterator node = nodeNameList.end();
@@ -53,6 +63,7 @@ Node *NodeList::addNodes(const QString &nodeName) {
         completeString = *node + "." + completeString;
         count++;
     }
+    emit dataChanged();
     return returnNode;
 }
 
@@ -247,19 +258,38 @@ void NodeList::reApplyFiltersTo(Node *node) {
     filterNode(node);
 }
 
-void NodeList::filterNone()
+void NodeList::clearAllFiltersAndEffects()
 {
     deleteFiltersAndEffects();
     setupFilterBox(0);
 }
 
-void NodeList::filterByName() {
-    deleteFiltersAndEffects();
+void NodeList::addNodesSlot(QString nodeName)
+{
+    addNodes(nodeName);
+}
 
+void NodeList::addNodesData(QString nodeName, DNSData nodeData, QString optionalLogMessage)
+{
+    addNodes(nodeName);
+    Node *theNode = node(nodeName);
+    theNode->addSubData(nodeData);
+    if (optionalLogMessage.length() > 0)
+        theNode->addLogMessage(optionalLogMessage);
+}
+
+Effect *NodeList::createDefaultEffect() {
     MultiEffect *effect = new MultiEffect();
     effect->addEffect(new SetZValue(-5));
     effect->addEffect(new SetAlphaEffect(64));
+    effect->addEffect(new SetNodeColoring(Qt::gray));
+    return effect;
+}
 
+void NodeList::filterByName() {
+    deleteFiltersAndEffects();
+
+    Effect *effect = createDefaultEffect();
     Filter *filter = new NotFilter(new NameFilter(""));
     setupFilterBox(filter);
 
@@ -274,9 +304,7 @@ void NodeList::filterByDataType()
 
     deleteFiltersAndEffects();
 
-    MultiEffect *effect = new MultiEffect();
-    effect->addEffect(new SetZValue(-5));
-    effect->addEffect(new SetAlphaEffect(64));
+    Effect *effect = createDefaultEffect();
 
     addFilterAndEffect(filter = new NotFilter(new TypeFilter("A")), effect);
 
@@ -290,9 +318,7 @@ void NodeList::filterBadToTop()
 
     deleteFiltersAndEffects();
 
-    MultiEffect *effect = new MultiEffect();
-    effect->addEffect(new SetZValue(-5));
-    effect->addEffect(new SetAlphaEffect(64));
+    Effect *effect = createDefaultEffect();
 
     addFilterAndEffect(filter = new NotFilter(new DNSSECStatusFilter(DNSData::FAILED)), effect);
 
@@ -300,6 +326,23 @@ void NodeList::filterBadToTop()
     applyFilters();
 }
 
+void NodeList::filterEditor()
+{
+    if (m_filterEditor) {
+        m_filterEditor->raise();
+        return;
+    }
+    m_filterEditor = new FilterEditorWindow(this);
+    m_filterEditor->show();
+    connect(m_filterEditor, SIGNAL(destroyed()), this, SLOT(closeEditor()));
+}
+
+void NodeList::closeEditor()
+{
+    m_filterEditor = 0;
+}
+
+
 void NodeList::setFilterBox(QHBoxLayout *filterBox)
 {
     m_filterBox = filterBox;
@@ -319,6 +362,7 @@ void NodeList::addFilterAndEffect(Filter *filter, Effect *effect)
 {
     m_filtersAndEffects.push_back(new FilterEffectPair(filter, effect));
     connect(filter, SIGNAL(filterChanged()), this, SLOT(applyFilters()));
+    connect(effect, SIGNAL(effectChanged()), this, SLOT(applyFilters()));
 }
 
 void NodeList::clearLayout(QLayout *layout) {
