@@ -46,6 +46,15 @@
 #include "DetailsViewer.h"
 #include "LogFilePicker.h"
 
+#include "LogWatcher.h"
+#include "NodeList.h"
+#ifdef WITH_PCAP
+#include "PcapWatcher.h"
+#endif
+#include "DNSData.h"
+
+#include "DNSResources.h"
+
 #include <QtGui>
 #include <qdebug.h>
 
@@ -87,13 +96,18 @@ void val_collect_logs(struct val_log *logp, int level, const char *buf)
     val_log_strings.push_back(buf);
 }
 
-GraphWidget::GraphWidget(QWidget *parent, QLineEdit *editor, const QString &fileName, QHBoxLayout *infoBox)
+GraphWidget::GraphWidget(QWidget *parent, QLineEdit *editor, QTabWidget *tabs, const QString &fileName, QHBoxLayout *infoBox)
     : QGraphicsView(parent), timerId(0), m_editor(editor),
       m_nodeScale(2), m_localScale(false), m_lockNodes(false), m_shownsec3(false),
       m_timer(0),
       m_layoutType(springyLayout), m_childSize(30), m_lookupType(1), m_animateNodeMovements(true),
+      m_updateLineEditAlways(false), m_autoValidateServFails(false),
       m_infoBox(infoBox), m_infoLabel(0), m_infoMoreButton(0), m_nodeInfoLabel(0), m_previousFileMenu(0), m_mapper(),
-      m_nodeList(new NodeList(this)), m_logWatcher(new LogWatcher(this)), m_legend(0)
+      m_nodeList(new NodeList(this)), m_logWatcher(new LogWatcher(this)), m_legend(0), m_tabs(tabs), m_useStraightValidationLines(false),
+      m_useToggledValidationBoxes(false), m_startingNode(ROOT_NODE_NAME)
+#ifdef WITH_PCAP
+    , m_pcapWatcher(new PcapWatcher())
+#endif
 {
     myScene = new QGraphicsScene(this);
     myScene->setItemIndexMethod(QGraphicsScene::NoIndex);
@@ -115,8 +129,8 @@ GraphWidget::GraphWidget(QWidget *parent, QLineEdit *editor, const QString &file
     m_infoLabel->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);
     m_infoBox->addWidget(m_infoMoreButton = new QPushButton("Details..."));
     connect(m_infoMoreButton, SIGNAL(clicked()), this, SLOT(moreInfoButton()));
-    m_infoMoreButton->hide();
-    m_nodeInfoLabel->hide();
+//    m_infoMoreButton->hide();
+//    m_nodeInfoLabel->hide();
 
     // m_logWatcher->parseLogFile();
 
@@ -126,12 +140,27 @@ GraphWidget::GraphWidget(QWidget *parent, QLineEdit *editor, const QString &file
     connect(m_logWatcher, SIGNAL(dataChanged()), this, SLOT(reLayout()));
     connect(m_nodeList, SIGNAL(dataChanged()), this, SLOT(reLayout()));
 
+    connect(this, SIGNAL(useStraightValidationLinesChanged()), this, SLOT(saveUseStraightValidationLinesPref()));
+    connect(this, SIGNAL(useToggledValidationBoxesChanged()), this, SLOT(saveUseToggledValidationBoxes()));
+
     val_log_add_cb(NULL, 99, &val_collect_logs);
 
     setPrefs();
 
     if (!fileName.isEmpty())
         m_logWatcher->parseLogFile(fileName);
+
+#ifdef WITH_PCAP
+    connect(m_pcapWatcher, SIGNAL(addNode(QString)), m_nodeList, SLOT(addNodesSlot(QString)));
+    connect(m_pcapWatcher, SIGNAL(addNodeData(QString, DNSData, QString)), m_nodeList, SLOT(addNodesData(QString,DNSData, QString)));
+    connect(m_pcapWatcher, SIGNAL(addNodeData(QString,DNSData,QString)), this, SLOT(doLookupFromServFail(QString,DNSData,QString)));
+    connect(this, SIGNAL(openPcapDevice()), m_pcapWatcher, SLOT(openDevice()));
+    m_pcapWatcher->start();
+#endif
+}
+
+void GraphWidget::resetStartingNode() {
+    setStartingNode(ROOT_NODE_NAME);
 }
 
 void GraphWidget::addItem(QGraphicsItem *newItem) {
@@ -162,7 +191,13 @@ void GraphWidget::scaleWindow() {
     qreal newscale = qMin(windowSize.width() / myScene->sceneRect().width(), windowSize.height() / myScene->sceneRect().width());
 
     // apply it
-    scale(.95 * newscale, .95 * newscale);
+    scale(1.2 * newscale, 1.2 * newscale);
+}
+
+void GraphWidget::doLookupFromServFail(QString nodeName, DNSData nodeData, QString optionalLogMessage) {
+    Q_UNUSED(optionalLogMessage)
+    if (autoValidateServFails())
+        doActualLookup(nodeName, DNSResources::RRNameToType(nodeData.recordType()));
 }
 
 void GraphWidget::doLookupFromLineEdit() {
@@ -170,6 +205,15 @@ void GraphWidget::doLookupFromLineEdit() {
     reLayout();
 }
 
+void GraphWidget::setLineEditValue(const QString &value) {
+    m_editor->setText(value);
+}
+
+void GraphWidget::maybeSetLineEditValue(const QString &value) {
+    if (m_updateLineEditAlways)
+        setLineEditValue(value);
+}
+
 void GraphWidget::itemMoved()
 {
     if (!m_lockNodes && !timerId)
@@ -350,17 +394,25 @@ int GraphWidget::layoutTreeNode(Node *node, int minX, int minY) {
 }
 
 void GraphWidget::layoutInCircles() {
+    const int childSize = 30;
+    QRectF sceneRect(-150,-150,150,150);
     m_layoutType = circleLayout;
-    QRectF rect;
-    layoutCircleNode(m_nodeList->node(ROOT_NODE_NAME), qreal(rect.left() + (rect.right() - rect.left())/2), qreal(rect.top() + (rect.top() - rect.bottom())/2), 0, 2*3.1415);
-
+    Node *startingNode = m_nodeList->node(m_startingNode);
+    layoutCircleNode(startingNode, 0, 0, 0, 2*3.1415, sceneRect, startingNode);
+    sceneRect.setBottom(sceneRect.bottom() + childSize);
+    sceneRect.setTop(sceneRect.top() - childSize);
+    sceneRect.setLeft(sceneRect.left() - childSize);
+    sceneRect.setRight(sceneRect.right() + childSize);
+    myScene->setSceneRect(sceneRect);
     // XXX: test growth size into borders
 }
 
-void GraphWidget::layoutCircleNode(Node *node, qreal startX, qreal startY, qreal startingDegrees, qreal maxDegrees) {
+void GraphWidget::layoutCircleNode(Node *node, qreal startX, qreal startY, qreal startingDegrees, qreal maxDegrees, QRectF &sceneRect, Node *upwardFromThis) {
     QSet<Node *> childNodes = node->children();
     const int childSize = 30;
     int numChildren = childNodes.count();
+    if (upwardFromThis && node->parent())
+        numChildren++;
 
     qreal degreesPerChild;
     if (numChildren > 0) {
@@ -368,9 +420,17 @@ void GraphWidget::layoutCircleNode(Node *node, qreal startX, qreal startY, qreal
 
         startingDegrees = startingDegrees - maxDegrees/2 + maxDegrees/(numChildren+1);
         foreach(Node *child, childNodes) {
+            if (upwardFromThis == 0 || upwardFromThis != child) {
+                qreal childX = startX + childSize*2*cos(startingDegrees);
+                qreal childY = startY + childSize*2*sin(startingDegrees);
+                layoutCircleNode(child, childX, childY, startingDegrees, degreesPerChild, sceneRect);
+                startingDegrees += degreesPerChild;
+            }
+        }
+        if (upwardFromThis && node->parent()){
             qreal childX = startX + childSize*2*cos(startingDegrees);
             qreal childY = startY + childSize*2*sin(startingDegrees);
-            layoutCircleNode(child, childX, childY, startingDegrees, degreesPerChild);
+            layoutCircleNode(node->parent(), childX, childY, startingDegrees, degreesPerChild, sceneRect, node);
             startingDegrees += degreesPerChild;
         }
     }
@@ -379,6 +439,8 @@ void GraphWidget::layoutCircleNode(Node *node, qreal startX, qreal startY, qreal
         return;
     }
 
+    sceneRect.setTopLeft(QPointF(qMin(sceneRect.left(), startX), qMin(sceneRect.top(), startY)));
+    sceneRect.setBottomRight(QPointF(qMax(sceneRect.right(), startX), qMax(sceneRect.bottom(), startY)));
     QPointF newpos(startX, startY);
     node->setNewPos(newpos);
     if (! m_animateNodeMovements)
@@ -404,10 +466,8 @@ void GraphWidget::doActualLookup(const QString &lookupString, int lookupType)
     busy();
 
     // perform the lookup
-    qDebug() << "here: " << lookupString;
     ret = val_res_query(NULL, lookupString.toUtf8(), 1,
                         lookupType, buf, sizeof(buf), &val_status);
-    qDebug() << "  result:" << ret;
 
     // do something with the results
     if (ret <= 0) {
@@ -433,7 +493,10 @@ void GraphWidget::doActualLookup(const QString &lookupString, int lookupType)
         } else {
             result = DNSData::FAILED;
         }
-        node->addSubData(DNSData("A", result));
+
+        QStringList dataThere = DNSResources::dnsDataToQStringList((char *) buf, ret);
+
+        node->addSubData(DNSData(DNSResources::typeToRRName(lookupType), result, dataThere));
     }
 
     QString lastInterestingString;
@@ -455,11 +518,12 @@ void GraphWidget::setShowNSEC3Records(bool newVal) {
 }
 
 void GraphWidget::unbusy() {
-    setCursor(Qt::ArrowCursor);
+    viewport()->setCursor(Qt::ArrowCursor);
 }
 
 void GraphWidget::busy() {
-    setCursor(Qt::WaitCursor);
+    viewport()->setCursor(Qt::WaitCursor);
+    viewport()->repaint();
 }
 
 void GraphWidget::setLockedNodes(bool newVal) {
@@ -514,8 +578,8 @@ void GraphWidget::setInfo(Node *node) {
 
 void GraphWidget::moreInfoButton() {
     if (m_nodeList->selectedNode()) {
-        DetailsViewer viewer(m_nodeList->selectedNode());
-        viewer.exec();
+        Node *node = m_nodeList->selectedNode();
+        node->displayDetailsMenu(m_infoMoreButton->mapToGlobal(QPoint(0,0)));
     }
 }
 
@@ -597,6 +661,13 @@ void GraphWidget::setPreviousFileList(QMenu *menu) {
     m_previousFileMenu = menu;
 }
 
+#ifdef WITH_PCAP
+PcapWatcher *GraphWidget::pcapWatcher()
+{
+    return m_pcapWatcher;
+}
+#endif
+
 void GraphWidget::openLogFile() {
     selectAndOpenLogFile();
 }
@@ -635,25 +706,60 @@ void GraphWidget::setPrefs()
     m_nodeList->setEnableMaxTime(settings.value("enableMaxNodes", false).toBool());
 
     m_animateNodeMovements = settings.value("animateNodes", true).toBool();
+    m_autoValidateServFails = settings.value("autoValidateServFails", false).toBool();
+    m_updateLineEditAlways = settings.value("updateLineEdit", false).toBool();
+
+    setUseStraightValidationLines(settings.value("useStraightValidationLines", false).toBool());
+    setUseToggledValidationBoxes(settings.value("useToggledValidationBoxes", false).toBool());
+}
+
+void GraphWidget::saveUseStraightValidationLinesPref() {
+    QSettings settings("DNSSEC-Tools", "dnssec-nodes");
+    settings.setValue("useStraightValidationLines", useStraightValidationLines());
+}
+
+void GraphWidget::saveUseToggledValidationBoxes() {
+    QSettings settings("DNSSEC-Tools", "dnssec-nodes");
+    settings.setValue("useToggledValidationBoxes", useToggledValidationBoxes());
 }
 
 void GraphWidget::setAnimateNodeMovements(bool newValue) {
     m_animateNodeMovements = newValue;
     QSettings settings("DNSSEC-Tools", "dnssec-nodes");
-    settings.setValue("animateNodes", true);
+    settings.setValue("animateNodes", newValue);
+}
+
+void GraphWidget::setUpdateLineEditAlways(bool newValue) {
+    m_updateLineEditAlways = newValue;
+    QSettings settings("DNSSEC-Tools", "dnssec-nodes");
+    settings.setValue("updateLineEdit", newValue);
+}
+
+void GraphWidget::setAutoValidateServFails(bool newValue) {
+    m_autoValidateServFails = newValue;
+    QSettings settings("DNSSEC-Tools", "dnssec-nodes");
+    settings.setValue("autoValidateServFails", newValue);
 }
 
 bool GraphWidget::animateNodeMovements() {
     return m_animateNodeMovements;
 }
 
+bool GraphWidget::updateLineEditAlways() {
+    return m_updateLineEditAlways;
+}
+
+bool GraphWidget::autoValidateServFails() {
+    return m_autoValidateServFails;
+}
+
 void GraphWidget::about()
 {
     QMessageBox msgBox;
     msgBox.setWindowTitle("About DNSSEC-Nodes");
     msgBox.setText("<img src=\":/icons/dnssec-nodes-64x64.png\" style=\"float: right\"/>"
                    "<h1>DNSSEC-Nodes</h1><p><i>A component of the DNSSEC-Tools Project</i></p><p>For more information, please visit http://www.dnssec-tools.org/.</i></p>"
-                   "<p>DNSSEC-Tools Version: 1.13</p>");
+                   "<p>DNSSEC-Tools Version: 2.0</p>");
     msgBox.setStandardButtons(QMessageBox::Close);
     msgBox.exec();
 }
