@@ -43,22 +43,29 @@
 #include <QPainter>
 #include <QMessageBox>
 #include <QStyleOption>
+#include <QTextEdit>
+#include <QMenu>
+#include <QToolTip>
 #include <qdebug.h>
 
 #include "edge.h"
 #include "node.h"
 #include "graphwidget.h"
 #include "DetailsViewer.h"
+#include "ValidateViewWidgetHolder.h"
+#include "DetailsViewer.h"
 
 Node::Node(GraphWidget *graphWidget, const QString &nodeName, const QString &fqdn, int depth)
     : m_parent(0), graph(graphWidget), m_nodeName(nodeName.toLower()), m_fqdn(fqdn.toLower()), m_depth(depth),
-      m_subData(), m_accessCount(0), m_accessTime(0), m_resultCache(0), m_colorAlpha(255)
+      m_subData(), m_accessCount(0), m_accessTime(0), m_resultCache(0), m_colorAlpha(255), m_borderColor(Qt::black),
+      m_nodeColor(), m_nodeSize(20), m_detailsViewer(0)
 {
     setFlag(ItemIsMovable);
     setFlag(ItemSendsGeometryChanges);
     setCacheMode(DeviceCoordinateCache);
     setZValue(-1);
     setToolTip(fqdn);
+    setAcceptHoverEvents(true);
 }
 
 void Node::addEdge(Edge *edge)
@@ -157,11 +164,11 @@ QRectF Node::boundingRect() const
     // Add some extra space around the circle for easier touching with finger
     qreal adjust = 30;
     return QRectF( -10 - adjust, -10 - adjust,
-                  20 + adjust * 2, 20 + adjust * 2);
+                  m_nodeSize + adjust * 2, m_nodeSize + adjust * 2);
 #else
     qreal adjust = 2;
     return QRectF( -10 - adjust, -10 - adjust,
-                  23 + adjust, 23 + adjust);
+                  m_nodeSize + 3 + adjust, m_nodeSize + 3 + adjust);
 #endif
 }
 
@@ -195,6 +202,12 @@ QColor Node::getColorForStatus(int status) {
         color = Qt::green;
     else if (status & DNSData::IGNORE)
         color = Qt::magenta;
+    else if (status & DNSData::AD_VERIFIED)
+        color = QColor(Qt::darkGreen);
+    else if (status & DNSData::AUTHORATATIVE)
+        color = QColor(Qt::green).lighter();
+    else if (status & DNSData::SERVFAIL_RCODE)
+        color = QColor(Qt::red).lighter();
     else
         color = QColor(128,128,128);
 
@@ -204,7 +217,9 @@ QColor Node::getColorForStatus(int status) {
 }
 
 void Node::setupPainting(int status, const QStyleOptionGraphicsItem *option, QPainter *painter) {
-    QColor color = getColorForStatus(status);
+    QColor color = m_nodeColor;
+    if (! m_nodeColor.isValid()) // normally the case; only if they chose specific color via a filter
+        color = getColorForStatus(status);
 
     QRadialGradient gradient(-3, -3, 10);
     if (option && option->state & QStyle::State_Sunken) {
@@ -218,38 +233,82 @@ void Node::setupPainting(int status, const QStyleOptionGraphicsItem *option, QPa
     }
 
     painter->setBrush(gradient);
-    painter->setPen(QPen(Qt::black, 0));
+    m_borderColor.setAlpha(m_colorAlpha);
+    painter->setPen(QPen(m_borderColor));
+}
+
+void Node::setBorderColor(const QColor &color)
+{
+    m_borderColor = color;
+    update();
+}
+
+QColor Node::borderColor()
+{
+    return m_borderColor;
+}
+
+void Node::setNodeColor(const QColor &color)
+{
+    m_nodeColor = color;
+    update();
+}
+
+QColor Node::nodeColor()
+{
+    return m_nodeColor;
+}
+
+void Node::setNodeSize(const unsigned int nodeSize)
+{
+    m_nodeSize = nodeSize;
+    update();
+}
+
+unsigned int Node::nodeSize() {
+    return m_nodeSize;
 }
 
 void Node::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
 {
+    // draw the shadow elipse
+    QColor shadowColor(Qt::gray);
+    shadowColor.setAlpha(m_colorAlpha);
     painter->setPen(Qt::NoPen);
-    painter->setBrush(Qt::darkGray);
-    painter->drawEllipse(-7, -7, 20, 20);
+    painter->setBrush(shadowColor);
+    painter->drawEllipse(-7, -7, m_nodeSize, m_nodeSize);
 
+    // draw the data bubble
     if (m_subData.count() > 1) {
         int angleSegment = 360 * 16 / m_subData.count();
         int count = 0;
 
-        foreach (DNSData data, m_subData) {
-            setupPainting(data.DNSSECStatus(), option, painter);
+        foreach (DNSData *data, m_subData) {
+            setupPainting(data->DNSSECStatus(), option, painter);
 
-            painter->drawPie(QRectF(-7, -7, 20, 20), count * angleSegment, angleSegment);
+            painter->drawPie(QRectF(-10, -10, m_nodeSize, m_nodeSize), count * angleSegment, angleSegment);
 
             count++;
         }
     } else {
         setupPainting(m_resultCache, option, painter);
 
-        painter->drawEllipse(-10, -10, 20, 20);
+        painter->drawEllipse(-10, -10, m_nodeSize, m_nodeSize);
     }
 
 
     if (m_nodeName.length() > 0) {
+        // draw the shadow elipse
+
         QFont font = painter->font();
         font.setPointSize(6);
         painter->setFont(font);
-        painter->drawText(QRectF(-10, -5, 20, 20), m_nodeName);
+
+        QColor penColor(Qt::black);
+        penColor.setAlpha(m_colorAlpha);
+        painter->setPen(QPen(penColor));
+
+        painter->drawText(QRectF(-10, -5, m_nodeSize, m_nodeSize), m_nodeName);
     }
 }
 
@@ -268,16 +327,82 @@ QVariant Node::itemChange(GraphicsItemChange change, const QVariant &value)
     return QGraphicsItem::itemChange(change, value);
 }
 
+QMenu *Node::makePopupMenu() {
+    QMenu *menu = new QMenu();
+
+    if (!graph->updateLineEditAlways())
+        menu->addAction(QObject::tr("Copy Name To Lookup Line"));
+    menu->addAction(QObject::tr("Show Node Data"));
+    menu->addAction(QObject::tr("Show Log Entries"));
+    menu->addAction(QObject::tr("Center Map on This Node"));
+    QMenu *validateMenu = menu->addMenu(QObject::tr("Validate"));
+
+    QMap<QString, DNSData *>::const_iterator iter, end = m_subData.end();
+    for (iter = m_subData.begin(); iter != end; iter++) {
+        validateMenu->addAction(iter.key());
+    }
+    return menu;
+}
+
+void Node::displayDetailsMenu(QPoint where) {
+    QMenu *menu = makePopupMenu();
+    QAction *action = menu->exec(where);
+
+    QWidget *widget = 0;
+    QString tabLabel = "";
+    QString menuChoice;
+
+    if (!action)
+        return;
+
+    menuChoice = action->text();
+
+    if (menuChoice == QObject::tr("Show Log Entries")) {
+        //
+        // Log Message Viewer
+        //
+        widget = new QWidget();
+        QVBoxLayout *vbox = new QVBoxLayout();
+
+        widget->setLayout(vbox);
+
+        QTextEdit *textEdit = new QTextEdit("<p>" + logMessages().join("</p><p>") + "</p>");
+        textEdit->setReadOnly(true);
+        textEdit->setLineWrapMode(QTextEdit::NoWrap);
+        vbox->addWidget(textEdit);
+
+        tabLabel = QObject::tr("%1 Log").arg(fqdn());
+    } else if (menuChoice == QObject::tr("Show Node Data")) {
+        widget = m_detailsViewer = new DetailsViewer(this, graph, graph->tabs());
+        tabLabel = fqdn() + " Data";
+    } else if (menuChoice == QObject::tr("Copy Name To Lookup Line")) {
+        graph->setLineEditValue(fqdn());
+    } else if (menuChoice == QObject::tr("Center Map on This Node")) {
+        graph->setStartingNode(fqdn());
+        setNewPos(QPointF(0,0));
+        graph->reLayout();
+    } else {
+        tabLabel = fqdn() + "/" + menuChoice;
+        widget = new ValidateViewWidgetHolder(fqdn(), menuChoice, graph);
+    }
+
+    if (widget) {
+        graph->tabs()->addTab(widget, tabLabel);
+        graph->tabs()->setCurrentIndex(graph->tabs()->count()-1);
+    }
+    delete menu;
+}
+
 void Node::mousePressEvent(QGraphicsSceneMouseEvent *event)
 {
     update();
+    graph->maybeSetLineEditValue(fqdn());
     if (event->button() == Qt::MidButton) {
         // middle button moves
         event->setButton(Qt::LeftButton);
         QGraphicsItem::mousePressEvent(event);
     } else if (event->button() == Qt::RightButton) {
-        DetailsViewer lv(this);
-        lv.exec();
+        displayDetailsMenu(event->screenPos());
     } else {
         // everything else selects
         graph->setInfo(this);
@@ -290,6 +415,12 @@ void Node::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
     QGraphicsItem::mouseReleaseEvent(event);
 }
 
+void Node::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
+{
+    Q_UNUSED(event);
+    graph->setInfo(this);
+}
+
 void Node::addChild(Node *child)
 {
     m_children.insert(child);
@@ -329,10 +460,15 @@ QStringList Node::logMessages()
 void Node::addSubData(const DNSData &data)
 {
     if (!m_subData.contains(data.recordType())) {
-        m_subData.insert(data.recordType(), data);
+        DNSData *newData = new DNSData(data);
+        m_subData.insert(data.recordType(), newData);
+        newData->setNode(this);
+        if (m_detailsViewer) {
+            m_detailsViewer->addRow(data.recordType(), newData);
+        }
     } else {
         // merge in the other data with ours (internally this drops UNKNOWNS
-        m_subData[data.recordType()].addDNSSECStatus(data.DNSSECStatus());
+        m_subData[data.recordType()]->addDNSSECStatus(data.DNSSECStatus());
     }
     cacheDNSDataValidity();
 }
@@ -340,8 +476,8 @@ void Node::addSubData(const DNSData &data)
 QString Node::getSubData()
 {
     QString description;
-    foreach(DNSData data, m_subData) {
-        description += data.recordType() + "(" + QString::number(int(data.DNSSECStatus())) + "), ";
+    foreach(DNSData *data, m_subData) {
+        description += data->recordType() + "(" + QString::number(int(data->DNSSECStatus())) + "), ";
     }
     return description;
 }
@@ -349,8 +485,8 @@ QString Node::getSubData()
 void Node::cacheDNSDataValidity()
 {
     m_resultCache = 0;
-    foreach(DNSData data, m_subData) {
-        m_resultCache |= data.DNSSECStatus();
+    foreach(DNSData *data, m_subData) {
+        m_resultCache |= data->DNSSECStatus();
     }
 }
 
@@ -361,9 +497,10 @@ int Node::DNSSECValidity()
 
 void Node::setAlpha(int alpha) {
     m_colorAlpha = alpha;
+    update();
 }
 
-QMap<QString, DNSData>Node::getAllSubData()
+QMap<QString, DNSData *>Node::getAllSubData()
 {
     return m_subData;
 }
