@@ -26,7 +26,7 @@
  * SUCH DAMAGE.
  */
 /*
- * Copyright 2007-2012 SPARTA, Inc.  All rights reserved.
+ * Copyright 2007-2013 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 
@@ -72,8 +72,8 @@ extern          "C" {
 #ifndef HAVE_MEMORY_H
 #define HAVE_MEMORY_H 1
 #endif
-#ifndef HAVE_SHA_256
-#define HAVE_SHA_256 1
+#ifndef HAVE_SHA_2
+#define HAVE_SHA_2 1
 #endif
 #ifndef HAVE_STDINT_H
 #define HAVE_STDINT_H 1
@@ -245,6 +245,14 @@ typedef unsigned short     u_int16_t;
 #define u_long unsigned long 
 #endif
 
+#ifndef HAVE_SIZE_T
+#define size_t unsigned int
+#endif
+
+#ifndef HAVE_SSIZE_T
+#define ssize_t int
+#endif
+
 #ifndef HAVE_SNPRINTF 
 #define snprintf _snprintf
 #endif
@@ -466,7 +474,7 @@ typedef struct {
 
 #endif
 
-#if !defined(HAVE_RESOLV_H) || defined(eabi) || defined(ANDROID) || defined(__OpenBSD__)
+#if !defined(HAVE_RESOLV_H) || defined(eabi) || defined(ANDROID) 
 #define RES_INIT        0x00000001      /* address initialized */
 #define RES_DEBUG       0x00000002      /* print debug messages */
 #define RES_AAONLY      0x00000004      /* authoritative answers only (!IMPL)*/
@@ -660,6 +668,9 @@ typedef enum __ns_rcode {
 #if !HAVE_DECL_NS_T_NSEC
 #define ns_t_nsec     47
 #endif
+#if !HAVE_DECL_NS_T_TLSA
+#define ns_t_tlsa     52 
+#endif
 
 #ifdef LIBVAL_NSEC3
 #if !HAVE_DECL_NS_T_NSEC3
@@ -685,7 +696,8 @@ typedef enum __ns_rcode {
 #endif /* HAVE_NS_R_BADSIG */
 
 /* eabi = android */
-#if !defined(HAVE_ARPA_NAMESER_H) || defined(eabi) || defined(ANDROID)
+/* OpenBSD has arpa/nameser.h, but it doesn't define ns_msg */
+#if !defined(HAVE_ARPA_NAMESER_H) || defined(eabi) || defined(ANDROID) ||defined(__OpenBSD__)
 /*
  * Define constants based on RFC 883, RFC 1034, RFC 1035
  */
@@ -886,6 +898,10 @@ int	ns_sprintrrf(const u_char *, size_t, const char *,
 		ns_class, ns_type, u_long, const u_char *,
 		size_t, const char *, const char *,
 		char *, size_t);
+int	ns_sprintrrf_data(const u_char *, size_t, const char *,
+		ns_class, ns_type, u_long, const u_char *,
+		size_t, const char *,
+		char *, size_t);
 int	ns_initparse(const u_char *, int, ns_msg *);
 int	ns_format_ttl(u_long, char *, size_t);
 int	ns_parse_ttl(const char *, u_long *);
