@@ -89,7 +89,8 @@ bind_to_random_source(int af, SOCKET s)
 
     u_int16_t next_port, start_port;
 
-    memset(&ea_source, 0, sizeof(ea_source));
+    memset(&ea_source, 0, sizeof(struct sockaddr_storage));
+
     if (af == AF_INET) {
         sa4->sin_family = AF_INET;
         sa4->sin_addr.s_addr = htonl(INADDR_ANY);
@@ -109,6 +110,7 @@ bind_to_random_source(int af, SOCKET s)
     next_port = start_port;
 
     do {
+        memset(&ea_source, 0, sizeof(ea_source));
         if (af == AF_INET) {
             sa4->sin_port = htons(next_port);
             sa = (struct sockaddr *) sa4;
@@ -238,7 +240,6 @@ res_ea_init(u_char * signed_query, size_t signed_length,
     memset(temp, 0x0, sizeof(struct expected_arrival));
     temp->ea_socket = INVALID_SOCKET;
     temp->ea_ns = ns;
-    temp->ea_edns0_size = ns->ns_edns0_size;
     temp->ea_which_address = 0;
     temp->ea_using_stream = FALSE;
     temp->ea_signed = signed_query;
@@ -252,42 +253,74 @@ res_ea_init(u_char * signed_query, size_t signed_length,
     return temp;
 }
 
+/*
+ * give up on current response for this address.
+ * set up next try to try address again.
+ */
 void
-res_io_cancel_remaining_attempts(struct expected_arrival *ea)
+res_io_retry_source(struct expected_arrival *ea)
 {
-    if (ea->ea_socket != INVALID_SOCKET) {
-        CLOSESOCK(ea->ea_socket);
-        ea->ea_socket = INVALID_SOCKET;
-    }
-    ea->ea_remaining_attempts = -1;
-}
+    res_log(NULL, LOG_DEBUG, "libsres: ""retry source ea %p", ea);
+    res_print_ea(ea);
 
-void
-res_io_cancel_source(struct expected_arrival *ea)
-{
     /* close socket */
     if (ea->ea_socket != INVALID_SOCKET) {
         CLOSESOCK(ea->ea_socket);
         ea->ea_socket = INVALID_SOCKET;
     }
 
-    /* no more retries */
-    ea->ea_remaining_attempts = -1;
+    /* bump retry time to current time */
+    gettimeofday(&ea->ea_next_try, NULL);
+}
+
+/*
+ * give up on current address for this nameserver.
+ * set cancel time, which will trigger move to next address.
+ */
+void
+res_io_reset_source(struct expected_arrival *ea)
+{
+    res_log(NULL, LOG_DEBUG, "libsres: ""reset source ea %p", ea);
+    res_print_ea(ea);
+
+    /* close socket */
+    if (ea->ea_socket != INVALID_SOCKET) {
+        CLOSESOCK(ea->ea_socket);
+        ea->ea_socket = INVALID_SOCKET;
+    }
 
     /* bump cancel time to current time */
     gettimeofday(&ea->ea_cancel_time, NULL);
 }
 
+/*
+ * give up on all addresses for this nameserver
+ * set remaining retries to -1; no more addresses will be tried
+ */
+void
+res_io_cancel_source(struct expected_arrival *ea)
+{
+    res_log(NULL, LOG_DEBUG, "libsres: ""canceling source ea %p", ea);
+    res_print_ea(ea);
+
+    /* close socket */
+    if (ea->ea_socket != INVALID_SOCKET) {
+        CLOSESOCK(ea->ea_socket);
+        ea->ea_socket = INVALID_SOCKET;
+    }
+
+    /* bump cancel time to current time */
+    gettimeofday(&ea->ea_cancel_time, NULL);
+
+    /* no more retries */
+    ea->ea_remaining_attempts = -1;
+}
+
 void
 res_io_cancel_all_remaining_attempts(struct expected_arrival *ea)
 {
-    for ( ; ea; ea = ea->ea_next) {
-        if (ea->ea_socket != INVALID_SOCKET) {
-            CLOSESOCK(ea->ea_socket);
-            ea->ea_socket = INVALID_SOCKET;
-        }
-        ea->ea_remaining_attempts = -1;
-    }
+    for ( ; ea; ea = ea->ea_next)
+        res_io_cancel_source(ea);
 }
 
 int
@@ -346,7 +379,7 @@ res_io_send(struct expected_arrival *shipit)
         /* Set the source port */
         if (0 != bind_to_random_source(af, shipit->ea_socket)) {
             /* error */
-            CLOSESOCK(shipit->ea_socket);
+            res_io_retry_source(shipit);
             return SR_IO_SOCKET_ERROR;
         }
 
@@ -371,8 +404,7 @@ res_io_send(struct expected_arrival *shipit)
             res_log(NULL, LOG_ERR,
                     "libsres: ""Closing socket %d, connect errno = %d",
                     shipit->ea_socket, errno);
-            CLOSESOCK(shipit->ea_socket);
-            shipit->ea_socket = INVALID_SOCKET;
+            res_io_reset_source(shipit);
             return SR_IO_SOCKET_ERROR;
         }
     }
@@ -391,15 +423,13 @@ res_io_send(struct expected_arrival *shipit)
         if ((bytes_sent =
              send(shipit->ea_socket, (const char *)&length_n, sizeof(length_n), 0))
             == SOCKET_ERROR) {
-            CLOSESOCK(shipit->ea_socket);
-            shipit->ea_socket = INVALID_SOCKET;
+            res_io_reset_source(shipit);
             return SR_IO_SOCKET_ERROR;
         }
 
 
         if (bytes_sent != sizeof(length_n)) {
-            CLOSESOCK(shipit->ea_socket);
-            shipit->ea_socket = INVALID_SOCKET;
+            res_io_reset_source(shipit);
             return SR_IO_SOCKET_ERROR;
         }
     }
@@ -410,8 +440,7 @@ res_io_send(struct expected_arrival *shipit)
         res_log(NULL, LOG_ERR, "libsres: "
                 "Closing socket %d, sending %d bytes failed (rc %d)",
                 shipit->ea_socket, shipit->ea_signed_length, bytes_sent);
-        CLOSESOCK(shipit->ea_socket);
-        shipit->ea_socket = INVALID_SOCKET;
+        res_io_reset_source(shipit);
         return SR_IO_SOCKET_ERROR;
     }
 
@@ -503,7 +532,7 @@ res_nsfallback_ea(struct expected_arrival *ea, struct timeval *closest_event,
     }
 
     if (server) {
-        for(;temp;temp=temp->ea_next) {
+        for(;temp && temp->ea_ns;temp=temp->ea_next) {
             //res_print_ea(temp);
             /** match name, then look for address */
             if (namecmp(server->ns_name_n, temp->ea_ns->ns_name_n) != 0)
@@ -531,13 +560,13 @@ res_nsfallback_ea(struct expected_arrival *ea, struct timeval *closest_event,
 
     res_log(NULL, LOG_DEBUG, "libsres: ""ea %p attempting ns fallback", temp);
 
-    old_size = temp->ea_edns0_size;
+    old_size = temp->ea_ns->ns_edns0_size;
     if ((temp->ea_ns->ns_options & SR_QUERY_VALIDATING_STUB_FLAGS) && 
-        (temp->ea_edns0_size > 0)) {
+        (temp->ea_ns->ns_edns0_size > 0)) {
         for (i = 0; i < sizeof(edns0_fallback); i++) {
-            if (temp->ea_edns0_size > edns0_fallback[i]) {
+            if (temp->ea_ns->ns_edns0_size > edns0_fallback[i]) {
                 /* try using a lower edns0 value */
-                temp->ea_edns0_size = edns0_fallback[i];
+                temp->ea_ns->ns_edns0_size = edns0_fallback[i];
                 if (edns0_fallback[i] == 0) {
                     /* try without EDNS0 */
                     res_log(NULL, LOG_DEBUG, "libsres: "
@@ -554,7 +583,7 @@ res_nsfallback_ea(struct expected_arrival *ea, struct timeval *closest_event,
     if (temp->ea_remaining_attempts == 0) {
         res_log(NULL, LOG_DEBUG, "libsres: "
                 "fallback already exhausted edns retries");
-        res_io_cancel_source(temp);
+        res_io_reset_source(temp);
         if (res_io_are_all_finished(ea))
             return -1;
         return 0;
@@ -590,7 +619,7 @@ res_nsfallback_ea(struct expected_arrival *ea, struct timeval *closest_event,
     res_log(NULL, LOG_INFO, "libsres: "
             "ns fallback for {%s %s(%d) %s(%d)}, edns0 size %d > %d",
             name, p_class(class_h), class_h, p_type(type_h), type_h,
-            old_size, temp->ea_edns0_size);
+            old_size, temp->ea_ns->ns_edns0_size);
 
     return 1;
 }
@@ -611,7 +640,6 @@ res_io_next_address(struct expected_arrival *ea,
             ea->ea_socket = INVALID_SOCKET;
         }
         ea->ea_which_address++;
-        ea->ea_edns0_size = ea->ea_ns->ns_edns0_size;
         ea->ea_remaining_attempts = ea->ea_ns->ns_retry+1;
         set_alarms(ea, 0, res_get_timeout(ea->ea_ns));
         res_log(NULL, LOG_INFO,
@@ -620,9 +648,9 @@ res_io_next_address(struct expected_arrival *ea,
         struct expected_arrival *next_server = ea->ea_next;
 
         /*
-         * cancel this source 
+         * tried all addresses, cancel this source 
          */
-        res_io_cancel_remaining_attempts(ea);
+        res_io_cancel_source(ea);
         res_log(NULL, LOG_INFO, "libsres: ""%s", no_more_str);
 
         if (next_server && next_server->ea_remaining_attempts &&
@@ -1223,18 +1251,39 @@ res_io_get_a_response(struct expected_arrival *ea_list, u_char ** answer,
 size_t
 complete_read(SOCKET sock, u_char *field, size_t length)
 {
-    size_t             bytes;
+    ssize_t             bytes;
     size_t             bytes_read = 0;
+#if 0
+#ifdef MSG_DONTWAIT
+    int                flags = MSG_DONTWAIT;
+#else
+    int                flags =0;
+#endif
+#endif
+    int                flags =0;/* XXX For now don't enable MSG_DONTWAIT */
+
     memset(field, '\0', length);
 
     do {
-        bytes = recv(sock, field + bytes_read, length - bytes_read, 0);
-        if (bytes == SOCKET_ERROR) {
-            bytes_read = -1;
-            return -1;
+        bytes = recv(sock, field + bytes_read, length - bytes_read, flags);
+        if (bytes < 0) {
+            break;
         }
         bytes_read += bytes;
-    } while (bytes_read < length && bytes != 0);
+    } while (bytes_read < length && bytes > 0);
+
+    if (0 == bytes)
+        res_log(NULL, LOG_INFO, "libsres: ""socket shutdown.", socket);
+    if (bytes_read != length) {
+        res_log(NULL, LOG_INFO,
+                "libsres: ""incomplete read on socket %d; read %d of %d\n",
+                sock, bytes_read, length);
+        if (bytes == SOCKET_ERROR) {
+            res_log(NULL, LOG_INFO, "libsres: ""  errno %d %s.", errno,
+                    strerror(errno));
+            return -1;
+        }
+    }
 
     return bytes_read;
 }
@@ -1250,8 +1299,10 @@ res_io_read_tcp(struct expected_arrival *arrival)
      */
     if (complete_read(arrival->ea_socket, (u_char *)&len_n, sizeof(len_n))
         != sizeof(len_n)) {
-        CLOSESOCK(arrival->ea_socket);
-        arrival->ea_socket = INVALID_SOCKET;
+        /*
+         * reset this source
+         */
+        res_io_reset_source(arrival);
         return SR_IO_SOCKET_ERROR;
     }
 
@@ -1262,8 +1313,10 @@ res_io_read_tcp(struct expected_arrival *arrival)
      */
     arrival->ea_response = (u_char *) MALLOC(len_h * sizeof(u_char));
     if (arrival->ea_response == NULL) {
-        CLOSESOCK(arrival->ea_socket);
-        arrival->ea_socket = INVALID_SOCKET;
+        /*
+         * retry this source 
+         */
+        res_io_retry_source(arrival);
         return SR_IO_MEMORY_ERROR;
     }
 
@@ -1275,9 +1328,9 @@ res_io_read_tcp(struct expected_arrival *arrival)
         arrival->ea_response = NULL;
         arrival->ea_response_length = 0;
         /*
-         * Cancel this source 
+         * reset this source 
          */
-        res_io_cancel_remaining_attempts(arrival);
+        res_io_reset_source(arrival);
         return SR_IO_SOCKET_ERROR;
     }
     return SR_IO_UNSET;
@@ -1305,6 +1358,9 @@ res_io_read_udp(struct expected_arrival *arrival)
     arrival->ea_response = (u_char *) MALLOC(bytes_waiting * sizeof(u_char));
     if (NULL == arrival->ea_response)
         return SR_IO_MEMORY_ERROR;
+    memset(arrival->ea_response, 0, bytes_waiting * sizeof(u_char));
+
+    memset(&from, 0, sizeof(from));
 
 #ifdef MSG_DONTWAIT
     flags = MSG_DONTWAIT;
@@ -1360,12 +1416,9 @@ res_io_read_udp(struct expected_arrival *arrival)
 
   error:
     /*
-     * Cancel this source 
+     * reset this source 
      */
-    res_io_cancel_source(arrival);
-    res_log(NULL, LOG_INFO, "libsres: ""Closing socket %d, %s",
-            arrival->ea_socket, (ret_val == 0) ?
-            "socket shutdown" : "read_udp failed");
+    res_io_reset_source(arrival);
 
   allow_retry:
     FREE(arrival->ea_response);
@@ -1445,7 +1498,7 @@ res_switch_all_to_tcp_tid(int tid)
 int
 res_io_read(fd_set * read_descriptors, struct expected_arrival *ea_list)
 {
-    int             handled = 0;
+    int             handled = 0, rc;
     struct expected_arrival *arrival;
 
     res_log(NULL,LOG_DEBUG,"libsres: "" res_io_read ea %p", ea_list);
@@ -1471,16 +1524,16 @@ res_io_read(fd_set * read_descriptors, struct expected_arrival *ea_list)
 
             if (arrival->ea_using_stream) {
                 /** Use TCP */
-                if (res_io_read_tcp(arrival) == SR_IO_SOCKET_ERROR)
-                    continue;
+                rc = res_io_read_tcp(arrival);
             } else {
                 /** Use UDP */
-                if (res_io_read_udp(arrival) == SR_IO_SOCKET_ERROR)
-                    continue;
+                rc = res_io_read_udp(arrival);
             }
-            res_log(NULL, LOG_DEBUG, "libsres: ""Read %zd byptes via %s",
+            res_log(NULL, LOG_DEBUG, "libsres: ""Read %zd bytes via %s",
                     arrival->ea_response_length,
                        arrival->ea_using_stream ? "TCP" : "UDP");
+            if (SR_IO_UNSET != rc)
+                continue;
 
             /*
              * Make sure this is the query we want (buffer id's match).
