@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2013 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  *
  * Author: Abhijit Hayatnagarkar
@@ -18,6 +18,10 @@
 #include "val_parse.h"
 #include "val_context.h"
 
+#ifndef  INADDR_LOOPBACK
+# define INADDR_LOOPBACK    0x7f000001
+#endif
+
 /* 
  * Free the addrinfo structure that we have allocated 
  */
@@ -411,6 +415,10 @@ get_addrinfo_from_etc_hosts(val_context_t * ctx,
         //val_log(ctx, LOG_DEBUG, "}");
 
         memset(ainfo, 0, sizeof(struct addrinfo));
+        memset(&sa, 0, sizeof(sa));
+#ifdef VAL_IPV6
+        memset(&sa6, 0, sizeof(sa6));
+#endif
 
         /*
          * Check if the address is an IPv4 address 
@@ -567,7 +575,7 @@ get_addrinfo_from_result(const val_context_t * ctx,
             canonname = result->val_ans_name;
 
             /*
-             * Loop for each rr in the linked list of val_rr_rec structures 
+             * Loop for each rr in the linked list 
              */
             while (rr != NULL) {
                 struct addrinfo *ainfo = NULL;
@@ -590,6 +598,7 @@ get_addrinfo_from_result(const val_context_t * ctx,
                         return EAI_MEMORY;
                     }
                     val_log(ctx, LOG_DEBUG, "get_addrinfo_from_result(): rrset of type A found");
+                    memset(saddr4, 0, sizeof(struct sockaddr_in));
                     saddr4->sin_family = AF_INET;
                     ainfo->ai_family = AF_INET;
                     ainfo->ai_addrlen = sizeof(struct sockaddr_in);
@@ -609,6 +618,7 @@ get_addrinfo_from_result(const val_context_t * ctx,
                         return EAI_MEMORY;
                     }
                     val_log(ctx, LOG_DEBUG, "get_addrinfo_from_result(): rrset of type AAAA found");
+                    memset(saddr6, 0, sizeof(struct sockaddr_in6));
                     saddr6->sin6_family = AF_INET6;
                     ainfo->ai_family = AF_INET6;
                     ainfo->ai_addrlen = sizeof(struct sockaddr_in6);
@@ -715,7 +725,7 @@ get_addrinfo_from_dns(val_context_t * ctx,
     struct addrinfo *ainfo = NULL;
     const struct addrinfo *hints;
     struct addrinfo default_hints;
-    int    ret = EAI_FAIL;
+    int    ret = EAI_FAIL, have4 = 1, have6 = 1;
 
     val_log(ctx, LOG_DEBUG, "get_addrinfo_from_dns() called");
 
@@ -743,13 +753,24 @@ get_addrinfo_from_dns(val_context_t * ctx,
         *val_status = VAL_UNTRUSTED_ANSWER;
         return EAI_NONAME; 
     }
+
+#ifdef AI_ADDRCONFIG
+    if (hints->ai_flags & AI_ADDRCONFIG) {
+        have4 = val_context_ip4(ctx);
+        have6 = val_context_ip6(ctx);
+    }
+#endif
     
     /*
      * Check if we need to return IPv4 addresses based on the hints 
      */
-    if (hints->ai_family == AF_UNSPEC || hints->ai_family == AF_INET) {
-
-        val_log(ctx, LOG_DEBUG, "get_addrinfo_from_dns(): checking for A records");
+    if ((hints->ai_family == AF_UNSPEC || hints->ai_family == AF_INET) 
+#ifdef AI_ADDRCONFIG
+        && (have4 != 0)
+#endif
+        ) {
+        val_log(ctx, LOG_DEBUG,
+                "get_addrinfo_from_dns(): checking for A records");
 
         if ((VAL_NO_ERROR == 
                     val_get_rrset(ctx, nodename, ns_c_in, ns_t_a, 0, &results)) 
@@ -758,8 +779,8 @@ get_addrinfo_from_dns(val_context_t * ctx,
             ret = get_addrinfo_from_result(ctx, results, servname,
                                          hints, &ainfo, val_status);
 
-            val_log(ctx, LOG_DEBUG, 
-                    "get_addrinfo_from_dns(): get_addrinfo_from_result() returned=%d with val_status=%d",
+            val_log(ctx, LOG_DEBUG, "get_addrinfo_from_dns(): "
+                    "get_addrinfo_from_result() returned=%d with val_status=%d",
                     ret, *val_status);
 
             val_free_answer_chain(results);
@@ -771,18 +792,23 @@ get_addrinfo_from_dns(val_context_t * ctx,
     /*
      * Check if we need to return IPv6 addresses based on the hints 
      */
-    if (hints->ai_family == AF_UNSPEC || hints->ai_family == AF_INET6) {
+    if ((hints->ai_family == AF_UNSPEC || hints->ai_family == AF_INET6)
+#ifdef AI_ADDRCONFIG
+        && (have6 != 0)
+#endif
+        ) {
 
-        val_log(ctx, LOG_DEBUG, "get_addrinfo_from_dns(): checking for AAAA records");
+        val_log(ctx, LOG_DEBUG,
+                "get_addrinfo_from_dns(): checking for AAAA records");
         
         if ((VAL_NO_ERROR == 
-                    val_get_rrset(ctx, nodename, ns_c_in, ns_t_aaaa, 0, &results)) 
-                && results) {
+             val_get_rrset(ctx, nodename, ns_c_in, ns_t_aaaa, 0, &results)) 
+            && results) {
             ret = get_addrinfo_from_result(ctx, results, servname,
                                          hints, &ainfo, val_status);
 
-            val_log(ctx, LOG_DEBUG, 
-                    "get_addrinfo_from_dns(): get_addrinfo_from_result() returned=%d with val_status=%d",
+            val_log(ctx, LOG_DEBUG, "get_addrinfo_from_dns(): "
+                    "get_addrinfo_from_result() returned=%d with val_status=%d",
                     ret, *val_status);
 
             val_free_answer_chain(results);
@@ -879,6 +905,11 @@ _getaddrinfo_local(val_context_t * ctx, const char *nodename,
         nname = localhost4;
     }
 
+    memset(&sa, 0, sizeof(sa));
+#ifdef VAL_IPV6
+    memset(&sa6, 0, sizeof(sa6));
+#endif
+
     /*
      * check for IPv4 addresses 
      */
@@ -1652,7 +1683,7 @@ val_getaddrinfo_submit(val_context_t * context, const char *nodename,
                        unsigned int val_gai_flags, val_gai_status **status)
 {
     val_gai_status        *vgai = NULL;
-    int                    vretval = VAL_NO_ERROR, rc;
+    int                    vretval = VAL_NO_ERROR, rc, have4 = 1, have6 = 1;
     struct addrinfo       *res = NULL;
     const struct addrinfo *hints;
     struct addrinfo        default_hints;
@@ -1708,10 +1739,21 @@ val_getaddrinfo_submit(val_context_t * context, const char *nodename,
     if (nodename)
         vgai->nodename = (char *)strdup(nodename);
 
+#ifdef AI_ADDRCONFIG
+    if (hints->ai_flags & AI_ADDRCONFIG) {
+        have4 = val_context_ip4(ctx);
+        have6 = val_context_ip6(ctx);
+    }
+#endif
+
     /*
      * Check if we need to return IPv4 addresses based on the hints
      */
-    if (hints->ai_family == AF_UNSPEC || hints->ai_family == AF_INET) {
+    if ((hints->ai_family == AF_UNSPEC || hints->ai_family == AF_INET)
+#ifdef AI_ADDRCONFIG
+        && (have4 != 0)
+#endif
+        ) {
 
         val_log(ctx, LOG_DEBUG,
                 "val_getaddrinfo_submit(): checking for A records");
@@ -1729,7 +1771,11 @@ val_getaddrinfo_submit(val_context_t * context, const char *nodename,
     /*
      * Check if we need to return IPv6 addresses based on the hints
      */
-    if (hints->ai_family == AF_UNSPEC || hints->ai_family == AF_INET6) {
+    if ((hints->ai_family == AF_UNSPEC || hints->ai_family == AF_INET6)
+#ifdef AI_ADDRCONFIG
+        && (have6 != 0)
+#endif
+        ) {
 
         val_log(ctx, LOG_DEBUG,
                 "val_getaddrinfo_submit(): checking for AAAA records");
