@@ -1,5 +1,5 @@
 /*
- * Copyright 2005-2012 SPARTA, Inc.  All rights reserved.
+ * Copyright 2005-2013 SPARTA, Inc.  All rights reserved.
  * See the COPYING file distributed with this software for details.
  */
 /*
@@ -1468,7 +1468,7 @@ read_next_val_config_file(val_context_t *ctx,
     int include_seen = 0;
     char *dnsval_filename = NULL;
     struct dnsval_list *dnsval_l;
-    int retval;
+    int retval = VAL_NO_ERROR;
     char *next_label;
     int done;
     char *env = NULL;
@@ -1511,7 +1511,12 @@ read_next_val_config_file(val_context_t *ctx,
 #ifdef HAVE_FLOCK
         memset(&fl, 0, sizeof(fl));
         fl.l_type = F_RDLCK;
-        fcntl(fd, F_SETLKW, &fl);
+        if (-1 == fcntl(fd, F_SETLK, &fl)) {
+            val_log(ctx, LOG_WARNING, 
+                "read_next_val_config_file(): Could not acquire shared lock on conf file: %s", 
+                dnsval_c->dnsval_conf);
+            goto err; 
+        }
 #endif
         if (0 != fstat(fd, &sb)) {
             val_log(ctx, LOG_ERR, 
@@ -1535,6 +1540,12 @@ read_next_val_config_file(val_context_t *ctx,
             retval = VAL_CONF_NOT_FOUND;
             goto err;
         }
+#ifdef HAVE_FLOCK
+        fl.l_type = F_UNLCK;
+        fcntl(fd, F_SETLK, &fl);
+#endif
+        close(fd);
+        fd = -1;
     }
 
     val_log(ctx, LOG_NOTICE, "read_next_val_config_file(): Reading validator policy from %s",
@@ -1771,14 +1782,6 @@ read_next_val_config_file(val_context_t *ctx,
     FREE(buf);
     buf = NULL;
 
-    if (fd > 0) {
-#ifdef HAVE_FLOCK
-        fl.l_type = F_UNLCK;
-        fcntl(fd, F_SETLKW, &fl);
-#endif
-        close(fd);
-        fd = -1;
-    }
     return VAL_NO_ERROR;
 
 err:
@@ -1794,7 +1797,7 @@ err:
     if (fd != -1) {
 #ifdef HAVE_FLOCK
         fl.l_type = F_UNLCK;
-        fcntl(fd, F_SETLKW, &fl);
+        fcntl(fd, F_SETLK, &fl);
 #endif
         close(fd);
     }
@@ -2096,8 +2099,14 @@ read_res_config_file(val_context_t * ctx)
 
     if (fd > 0) {
 #ifdef HAVE_FLOCK
+        memset(&fl, 0, sizeof(fl));
         fl.l_type = F_RDLCK;
-        fcntl(fd, F_SETLKW, &fl);
+        if (-1 == fcntl(fd, F_SETLK, &fl)) {
+            val_log(ctx, LOG_WARNING, 
+                "read_next_val_config_file(): Could not acquire shared lock on conf file: %s", 
+                resolv_config);
+            goto err;
+        }
 #endif
 
         if (0 != fstat(fd, &sb)) {
@@ -2120,6 +2129,13 @@ read_res_config_file(val_context_t * ctx)
             retval = VAL_CONF_NOT_FOUND;
             goto err;
         }
+#ifdef HAVE_FLOCK
+        fl.l_type = F_UNLCK;
+        fcntl(fd, F_SETLK, &fl);
+#endif
+        close(fd);
+        fd = -1;
+
         val_log(ctx, LOG_NOTICE, "read_res_config_file(): Reading resolver policy from %s", resolv_config);
 
     } else {
@@ -2240,13 +2256,6 @@ read_res_config_file(val_context_t * ctx)
     FREE(buf);
 
   empty:
-    if (fd > 0 ) {
-#ifdef HAVE_FLOCK
-        fl.l_type = F_UNLCK;
-        fcntl(fd, F_SETLKW, &fl);
-#endif
-        close(fd);
-    }
 
     /*
      * Check if we have root hints 
@@ -2265,6 +2274,14 @@ read_res_config_file(val_context_t * ctx)
 
     val_log(ctx, LOG_DEBUG, 
             "read_res_config_file(): Done reading resolver configuration");
+
+    if (fd != -1) {
+#ifdef HAVE_FLOCK
+        fl.l_type = F_UNLCK;
+        fcntl(fd, F_SETLK, &fl);
+#endif
+        close(fd);
+    }
     return VAL_NO_ERROR;
 
   err:
@@ -2278,7 +2295,7 @@ read_res_config_file(val_context_t * ctx)
     if (fd != -1) {
 #ifdef HAVE_FLOCK
         fl.l_type = F_UNLCK;
-        fcntl(fd, F_SETLKW, &fl);
+        fcntl(fd, F_SETLK, &fl);
 #endif
         close(fd);
     }
@@ -2334,8 +2351,14 @@ read_root_hints_file(val_context_t * ctx)
     if (NULL != root_hints && 
             (fd = open(root_hints, O_RDONLY)) > 0) {
 #ifdef HAVE_FLOCK
+        memset(&fl, 0, sizeof(fl));
         fl.l_type = F_RDLCK;
-        fcntl(fd, F_SETLKW, &fl);
+        if (-1 == fcntl(fd, F_SETLK, &fl)) {
+            val_log(ctx, LOG_WARNING, 
+                    "read_next_val_config_file(): Could not acquire shared lock on conf file: %s", 
+                    root_hints);
+            goto err;
+        }
 #endif
         if (0 != fstat(fd, &sb)) { 
             retval = VAL_CONF_NOT_FOUND;
@@ -2354,6 +2377,13 @@ read_root_hints_file(val_context_t * ctx)
             retval = VAL_CONF_NOT_FOUND;
             goto err;
         }
+#ifdef HAVE_FLOCK
+        fl.l_type = F_UNLCK;
+        fcntl(fd, F_SETLK, &fl);
+#endif
+        close(fd);
+        fd = -1;
+
         val_log(ctx, LOG_NOTICE, "read_root_hints_file(): Reading root hints from %s",
                 root_hints);
 
@@ -2486,6 +2516,7 @@ read_root_hints_file(val_context_t * ctx)
         if (type_h == ns_t_a) {
             struct sockaddr_in sa;
             size_t addrlen4 = sizeof(struct sockaddr_in);
+            memset(&sa, 0, sizeof(sa));
             if ((addrlen4 == addrlen4) && /* this is to remove unused variable warning */
                 (INET_PTON(AF_INET, token, ((struct sockaddr *)&sa), &addrlen4) != 1)) {
                 retval = VAL_CONF_PARSE_ERROR;
@@ -2497,6 +2528,7 @@ read_root_hints_file(val_context_t * ctx)
         } else if (type_h == ns_t_aaaa) {
             struct sockaddr_in6 sa6;
             size_t addrlen6 = sizeof(struct sockaddr_in6);
+            memset(&sa6, 0, sizeof(sa6));
             if ((addrlen6 == addrlen6) && /* this is to remove unused variable warning */
                 (INET_PTON(AF_INET6, token, ((struct sockaddr *)&sa6), &addrlen6) != 1)) {
                 val_log(ctx, LOG_INFO, 
@@ -2527,10 +2559,10 @@ read_root_hints_file(val_context_t * ctx)
             goto err;
         }
         if (type_h != ns_t_rrsig) {
-            /** Add this record to its chain of val_rr_rec's. */
+            /* Add this record to its chain. */
             retval = add_to_set(rr_set, rdata_len_h, rdata_n);
         } else {
-            /** Add this record to the sig of rrset_rec. */
+            /* Add this record's sig to its chain. */
             retval = add_as_sig(rr_set, rdata_len_h, rdata_n);
         }
         if (retval != VAL_NO_ERROR) {
@@ -2543,7 +2575,7 @@ read_root_hints_file(val_context_t * ctx)
 
     if (VAL_NO_ERROR !=
         (retval =
-         res_zi_unverified_ns_list(&ns_list, root_zone_n, root_info,
+         res_zi_unverified_ns_list(ctx, &ns_list, root_zone_n, root_info,
                                    &pending_glue))) {
 
         goto err;
@@ -2575,13 +2607,6 @@ read_root_hints_file(val_context_t * ctx)
     val_log(ctx, LOG_DEBUG, "read_root_hints_file(): Done reading root hints");
     FREE(buf);
 
-    if (fd > 0) {
-#ifdef HAVE_FLOCK
-        fl.l_type = F_UNLCK;
-        fcntl(fd, F_SETLKW, &fl);
-#endif
-        close(fd);
-    }
 
     return VAL_NO_ERROR;
 
@@ -2592,7 +2617,7 @@ read_root_hints_file(val_context_t * ctx)
     if (fd > 0) {
 #ifdef HAVE_FLOCK
         fl.l_type = F_UNLCK;
-        fcntl(fd, F_SETLKW, &fl);
+        fcntl(fd, F_SETLK, &fl);
 #endif
         close(fd);
     }
